@@ -18,9 +18,12 @@
  */
 #include "config.h"
 
+#define G_LOG_DOMAIN "GcalEditDialog"
+
+#include "gcal-date-selector.h"
+#include "gcal-debug.h"
 #include "gcal-edit-dialog.h"
 #include "gcal-time-selector.h"
-#include "gcal-date-selector.h"
 #include "gcal-utils.h"
 
 #include <libecal/libecal.h>
@@ -133,6 +136,18 @@ const GActionEntry action_entries[] =
   { "select-calendar", on_calendar_selected, "s" },
 };
 
+static gint
+sources_menu_sort_func (gconstpointer a,
+                        gconstpointer b)
+{
+  ESource *source1, *source2;
+
+  source1 = E_SOURCE (a);
+  source2 = E_SOURCE (b);
+
+  return g_ascii_strcasecmp (e_source_get_display_name (source1), e_source_get_display_name (source2));
+}
+
 static void
 fill_sources_menu (GcalEditDialog *dialog)
 {
@@ -145,18 +160,22 @@ fill_sources_menu (GcalEditDialog *dialog)
   list = gcal_manager_get_sources (dialog->manager);
   dialog->sources_menu = g_menu_new ();
 
+  list = g_list_sort (list, sources_menu_sort_func);
+
   for (aux = list; aux != NULL; aux = aux->next)
     {
       ESource *source;
       GMenuItem *item;
       GdkRGBA color;
+      cairo_surface_t *surface;
       GdkPixbuf *pix;
 
       source = E_SOURCE (aux->data);
 
       /* retrieve color */
       get_color_name_from_source (source, &color);
-      pix = get_circle_pixbuf_from_color (&color, 16);;
+      surface = get_circle_surface_from_color (&color, 16);
+      pix = gdk_pixbuf_get_from_surface (surface, 0, 0, 16, 16);
 
       /* menu item */
       item = g_menu_item_new (e_source_get_display_name (source), "select-calendar");
@@ -175,6 +194,7 @@ fill_sources_menu (GcalEditDialog *dialog)
 
       g_menu_append_item (dialog->sources_menu, item);
 
+      g_clear_pointer (&surface, cairo_surface_destroy);
       g_object_unref (pix);
       g_object_unref (item);
     }
@@ -197,6 +217,8 @@ on_calendar_selected (GSimpleAction *action,
   GList *aux;
   gchar *uid;
 
+  GCAL_ENTRY;
+
   self = GCAL_EDIT_DIALOG (user_data);
   list = gcal_manager_get_sources (self->manager);
 
@@ -212,24 +234,27 @@ on_calendar_selected (GSimpleAction *action,
       if (g_strcmp0 (e_source_get_uid (source), uid) == 0)
       {
         GdkRGBA color;
-        GdkPixbuf *pix;
+        cairo_surface_t *surface;
 
         /* retrieve color */
         get_color_name_from_source (source, &color);
 
-        pix = get_circle_pixbuf_from_color (&color, 16);
-        gtk_image_set_from_pixbuf (GTK_IMAGE (self->source_image), pix);
-        g_object_unref (pix);
+        surface = get_circle_surface_from_color (&color, 16);
+        gtk_image_set_from_surface (GTK_IMAGE (self->source_image), surface);
 
         self->selected_source = source;
 
         gtk_label_set_label (GTK_LABEL (self->subtitle_label), e_source_get_display_name (source));
+
+        g_clear_pointer (&surface, cairo_surface_destroy);
         break;
       }
     }
 
   g_free (uid);
   g_list_free (list);
+
+  GCAL_EXIT;
 }
 
 static void
@@ -303,13 +328,15 @@ sync_datetimes (GcalEditDialog *self,
   gboolean is_start;
   gint hour_to_add;
 
+  GCAL_ENTRY;
+
   is_start = (widget == self->start_time_selector || widget == self->start_date_selector);
   start = gcal_edit_dialog_get_date_start (self);
   end = gcal_edit_dialog_get_date_end (self);
 
   /* The date is valid, no need to update the fields */
   if (g_date_time_compare (end, start) >= 0)
-    goto out;
+    GCAL_GOTO (out);
 
   /*
    * If the user is changing the start date or time, we change the end
@@ -346,6 +373,8 @@ sync_datetimes (GcalEditDialog *self,
 out:
   g_clear_pointer (&start, g_date_time_unref);
   g_clear_pointer (&end, g_date_time_unref);
+
+  GCAL_EXIT;
 }
 
 static void
@@ -544,6 +573,8 @@ gcal_edit_dialog_finalize (GObject *object)
 {
   GcalEditDialog *dialog;
 
+  GCAL_ENTRY;
+
   dialog = GCAL_EDIT_DIALOG (object);
 
   g_clear_object (&dialog->action_group);
@@ -551,6 +582,8 @@ gcal_edit_dialog_finalize (GObject *object)
   g_clear_object (&dialog->event);
 
   G_OBJECT_CLASS (gcal_edit_dialog_parent_class)->finalize (object);
+
+  GCAL_EXIT;
 }
 
 static void
@@ -601,6 +634,8 @@ gcal_edit_dialog_action_button_clicked (GtkWidget *widget,
 {
   GcalEditDialog *dialog;
 
+  GCAL_ENTRY;
+
   dialog = GCAL_EDIT_DIALOG (user_data);
 
   if (widget == dialog->cancel_button || (widget == dialog->done_button && !dialog->writable))
@@ -692,7 +727,7 @@ gcal_edit_dialog_action_button_clicked (GtkWidget *widget,
                            dialog->event_is_new ? GCAL_RESPONSE_CREATE_EVENT : GCAL_RESPONSE_SAVE_EVENT);
     }
 
-  gcal_edit_dialog_set_event (dialog, NULL);
+  GCAL_EXIT;
 }
 
 static void
@@ -1037,13 +1072,15 @@ void
 gcal_edit_dialog_set_event (GcalEditDialog *dialog,
                             GcalEvent      *event)
 {
+  GCAL_ENTRY;
+
   g_return_if_fail (GCAL_IS_EDIT_DIALOG (dialog));
 
   if (g_set_object (&dialog->event, event))
     {
       GDateTime *date_start;
       GDateTime *date_end;
-      GdkPixbuf *pix;
+      cairo_surface_t *surface;
       ESource *source;
       const gchar *summary;
       gboolean all_day;
@@ -1052,7 +1089,7 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
 
       /* If we just set the event to NULL, simply send a property notify */
       if (!event)
-        goto out;
+        GCAL_GOTO (out);
 
       all_day = gcal_event_get_all_day (event);
       source = gcal_event_get_source (event);
@@ -1076,9 +1113,9 @@ gcal_edit_dialog_set_event (GcalEditDialog *dialog,
         gtk_entry_set_text (GTK_ENTRY (dialog->summary_entry), summary);
 
       /* dialog titlebar's title & subtitle */
-      pix = get_circle_pixbuf_from_color (gcal_event_get_color (event), 16);
-      gtk_image_set_from_pixbuf (GTK_IMAGE (dialog->source_image), pix);
-      g_object_unref (pix);
+      surface = get_circle_surface_from_color (gcal_event_get_color (event), 16);
+      gtk_image_set_from_surface (GTK_IMAGE (dialog->source_image), surface);
+      g_clear_pointer (&surface, cairo_surface_destroy);
 
       gtk_label_set_label (GTK_LABEL (dialog->subtitle_label), e_source_get_display_name (source));
 
@@ -1137,6 +1174,8 @@ out:
 
       dialog->setting_event = FALSE;
     }
+
+  GCAL_EXIT;
 }
 
 void
