@@ -17,13 +17,17 @@
  * with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GcalEventWidget"
+
 #include <string.h>
+#include <glib/gi18n.h>
 
 #include "gcal-event-widget.h"
 #include "gcal-utils.h"
 
-#define INTENSITY(c) ((c->red) * 0.30 + (c->green) * 0.59 + (c->blue) * 0.11)
-#define ICON_SIZE    16
+#define DESC_MAX_CHAR 200
+#define INTENSITY(c)  ((c->red) * 0.30 + (c->green) * 0.59 + (c->blue) * 0.11)
+#define ICON_SIZE     16
 
 struct _GcalEventWidget
 {
@@ -67,6 +71,93 @@ static guint signals[NUM_SIGNALS] = { 0, };
 G_DEFINE_TYPE_WITH_CODE (GcalEventWidget, gcal_event_widget, GTK_TYPE_WIDGET,
                          G_IMPLEMENT_INTERFACE (GTK_TYPE_ORIENTABLE, NULL));
 
+static gchar*
+get_visible_text (GtkWidget         *widget,
+                  GtkOrientation     orientation)
+{
+  GcalEventWidget *self;
+  gboolean is_ltr;
+  gchar *display_text;
+  gchar *escaped_text;
+
+  self = GCAL_EVENT_WIDGET (widget);
+  is_ltr = gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL;
+
+  /*
+   * Since we set markup labels, we have to escape text before
+   * using it, or we get empty event widgets.
+   */
+  escaped_text = g_markup_escape_text (gcal_event_get_summary (self->event), -1);
+
+  /*
+   * The text displayed in the widget includes the time when the event
+   * is timed, or not if the event is all day.
+   */
+  if (gcal_event_get_all_day (self->event))
+    {
+      display_text = escaped_text;
+    }
+  else
+    {
+      GDateTime *local_start_time;
+      gchar *start_time;
+      local_start_time = g_date_time_to_local (gcal_event_get_date_start (self->event));
+
+      if (self->clock_format_24h)
+        start_time = g_date_time_format (local_start_time, "%R");
+      else
+        start_time = g_date_time_format (local_start_time, "%I:%M %P");
+
+      /*
+       * When in horizontal, draw the elements sequentially; otherwise,
+       * draw the time in the first line, and the title in subsequent
+       * lines.
+       */
+      if (orientation == GTK_ORIENTATION_HORIZONTAL)
+        {
+          if (is_ltr)
+            display_text = g_strdup_printf ("(%s) %s", start_time, escaped_text);
+          else
+            display_text = g_strdup_printf ("%s (%s)", escaped_text, start_time);
+        }
+      else
+        {
+          display_text = g_strdup_printf ("<b>%s</b>\n%s", start_time, escaped_text);
+        }
+
+      g_clear_pointer (&local_start_time, g_date_time_unref);
+      g_clear_pointer (&escaped_text, g_free);
+      g_clear_pointer (&start_time, g_free);
+    }
+
+    return display_text;
+}
+
+static gint
+get_vertical_text_height (GtkWidget         *widget)
+{
+  GcalEventWidget *self;
+  GtkBorder border, padding;
+  GtkStyleContext *context;
+  PangoLayout *layout;
+  gint layout_height;
+  gchar* display_text;
+
+  self = GCAL_EVENT_WIDGET (widget);
+  context = gtk_widget_get_style_context (widget);
+  display_text = get_visible_text (widget, GTK_ORIENTATION_VERTICAL);
+  layout = gtk_widget_create_pango_layout (widget, display_text);
+  pango_layout_get_pixel_size (layout, NULL, &layout_height);
+  g_object_unref (layout);
+
+  gtk_style_context_get_border (context, gtk_style_context_get_state (context), &border);
+  gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
+
+  g_clear_pointer(&display_text, g_free);
+
+  return layout_height + padding.top + padding.bottom + border.top + border.bottom;
+}
+
 static void
 gcal_event_widget_update_style (GcalEventWidget *self)
 {
@@ -151,6 +242,157 @@ update_color (GcalEventWidget *self)
   g_free (color_str);
 }
 
+static void
+gcal_event_widget_set_event_tooltip (GcalEventWidget *self,
+                                     GcalEvent       *event)
+{
+  g_autoptr (GDateTime) tooltip_start, tooltip_end;
+  g_autofree gchar *start, *end, *escaped_summary;
+  GString *tooltip_mesg;
+  gboolean allday, multiday, is_ltr;
+  guint description_len;
+
+  tooltip_mesg = g_string_new (NULL);
+  escaped_summary = g_markup_escape_text (gcal_event_get_summary (event), -1);
+  g_string_append_printf (tooltip_mesg, "<b>%s</b>", escaped_summary);
+
+  tooltip_start = g_date_time_to_local (gcal_event_get_date_start (event));
+  tooltip_end = g_date_time_to_local (gcal_event_get_date_end (event));
+
+  allday = gcal_event_get_all_day (event);
+  multiday = gcal_event_is_multiday (event);
+
+  is_ltr = gtk_widget_get_direction (GTK_WIDGET (self)) != GTK_TEXT_DIR_RTL;
+
+  if (allday)
+    {
+      if (multiday)
+        {
+          start = g_date_time_format (tooltip_start, "%x");
+          end = g_date_time_format (tooltip_end, "%x");
+        }
+      else
+        {
+          start = g_date_time_format (tooltip_start, "%x");
+          end = NULL;
+        }
+    }
+  else
+    {
+      if (multiday)
+        {
+          if (self->clock_format_24h)
+            {
+              if (is_ltr)
+                {
+                  start = g_date_time_format (tooltip_start, "%x %R");
+                  end = g_date_time_format (tooltip_end, "%x %R");
+                }
+              else
+                {
+                  start = g_date_time_format (tooltip_start, "%R %x");
+                  end = g_date_time_format (tooltip_end, "%R %x");
+                }
+            }
+          else
+            {
+              if (is_ltr)
+                {
+                  start = g_date_time_format (tooltip_start, "%x I:%M %P");
+                  end = g_date_time_format (tooltip_end, "%x I:%M %P");
+                }
+              else
+                {
+                  start = g_date_time_format (tooltip_start, "%P %M:%I %x");
+                  end = g_date_time_format (tooltip_end, "%P %M:%I %x");
+                }
+            }
+        }
+      else
+        {
+          if (self->clock_format_24h)
+            {
+              if (is_ltr)
+                {
+                  start = g_date_time_format (tooltip_start, "%x, %R");
+                  end = g_date_time_format (tooltip_end, "%R");
+                }
+              else
+                {
+                  start = g_date_time_format (tooltip_start, "%R ,%x");
+                  end = g_date_time_format (tooltip_end, "%R");
+                }
+            }
+          else
+            {
+              if (is_ltr)
+                {
+                  start = g_date_time_format (tooltip_start, "%x, I:%M %P");
+                  end = g_date_time_format (tooltip_end, "I:%M %P");
+                }
+              else
+                {
+                  start = g_date_time_format (tooltip_start, "%P %M:%I ,%x");
+                  end = g_date_time_format (tooltip_end, "%P %M:%I");
+                }
+            }
+        }
+    }
+
+  if (allday && !multiday)
+    {
+      g_string_append_printf (tooltip_mesg,
+                              "\n%s",
+                              start);
+    }
+  else
+    {
+      g_string_append_printf (tooltip_mesg,
+                              "\n%s - %s",
+                              is_ltr ? start : end,
+                              is_ltr ? end : start);
+    }
+
+  /* Append event location */
+  if (g_utf8_strlen (gcal_event_get_location (event), -1) > 0)
+    {
+      g_autofree gchar *escaped_location;
+
+      escaped_location = g_markup_escape_text (gcal_event_get_location (event), -1);
+
+      g_string_append (tooltip_mesg, "\n\n");
+      g_string_append_printf (tooltip_mesg, "%s", escaped_location);
+    }
+
+  description_len = g_utf8_strlen (gcal_event_get_description (event), -1);
+
+  /* Truncate long descriptions at a white space and ellipsize */
+  if (description_len > 0)
+    {
+      g_autofree gchar *escaped_description;
+      GString *tooltip_desc;
+
+      tooltip_desc = g_string_new (gcal_event_get_description (event));
+
+      /* If the description is larger than DESC_MAX_CHAR, ellipsize it */
+      if (description_len > DESC_MAX_CHAR)
+        {
+          g_string_truncate (tooltip_desc, DESC_MAX_CHAR - 1);
+          g_string_append (tooltip_desc, "…");
+        }
+
+      escaped_description = g_markup_escape_text (tooltip_desc->str, -1);
+
+      g_string_append_printf (tooltip_mesg, "\n\n%s", escaped_description);
+
+      g_string_free (tooltip_desc, TRUE);
+    }
+
+  gtk_widget_set_tooltip_markup (GTK_WIDGET (self), tooltip_mesg->str);
+
+  g_string_free (tooltip_mesg, TRUE);
+}
+
 static void
 gcal_event_widget_set_event_internal (GcalEventWidget *self,
                                       GcalEvent       *event)
@@ -186,6 +428,8 @@ gcal_event_widget_set_event_internal (GcalEventWidget *self,
                             "notify::summary",
                             G_CALLBACK (gtk_widget_queue_draw),
                             self);
+
+  gcal_event_widget_set_event_tooltip (self, event);
 }
 
 static void
@@ -205,7 +449,7 @@ gcal_event_widget_init (GcalEventWidget *self)
                        GDK_BUTTON1_MASK,
                        NULL,
                        0,
-                       GDK_ACTION_COPY);
+                       GDK_ACTION_MOVE);
 
   gtk_drag_source_add_text_targets (widget);
 }
@@ -326,23 +570,24 @@ gcal_event_widget_get_preferred_height (GtkWidget *widget,
   GtkStyleContext *context;
   PangoLayout *layout;
   gint layout_height;
-  gint factor;
+  gchar* display_text;
 
   self = GCAL_EVENT_WIDGET (widget);
   context = gtk_widget_get_style_context (widget);
-  layout = gtk_widget_create_pango_layout (widget, NULL);
+  display_text = get_visible_text (widget, self->orientation);
+  layout = gtk_widget_create_pango_layout (widget, display_text);
   pango_layout_get_pixel_size (layout, NULL, &layout_height);
   g_object_unref (layout);
 
   gtk_style_context_get_border (context, gtk_style_context_get_state (context), &border);
   gtk_style_context_get_padding (context, gtk_style_context_get_state (context), &padding);
 
-  factor = self->orientation == GTK_ORIENTATION_HORIZONTAL ? 1 : 2;
-
   if (minimum != NULL)
-    *minimum = (factor * layout_height) + padding.top + padding.bottom + border.top + border.bottom;
+    *minimum = layout_height + padding.top + padding.bottom + border.top + border.bottom;
   if (natural != NULL)
-    *natural = (factor * layout_height) + padding.top + padding.bottom + border.top + border.bottom;
+    *natural = layout_height + padding.top + padding.bottom + border.top + border.bottom;
+
+  g_clear_pointer(&display_text, g_free);
 }
 
 static void
@@ -473,7 +718,6 @@ gcal_event_widget_draw (GtkWidget *widget,
   PangoLayout *layout;
   PangoFontDescription *font_desc;
   gchar *display_text;
-  gchar *escaped_text;
 
   self = GCAL_EVENT_WIDGET (widget);
   context = gtk_widget_get_style_context (widget);
@@ -488,51 +732,15 @@ gcal_event_widget_draw (GtkWidget *widget,
   gtk_render_background (context, cr, 0, 0, width, height);
   gtk_render_frame (context, cr, 0, 0, width, height);
 
-  /*
-   * Since we set markup labels, we have to escape text before
-   * using it, or we get empty event widgets.
-   */
-  escaped_text = g_markup_escape_text (gcal_event_get_summary (self->event), -1);
-
-  /*
-   * The text displayed in the widget includes the time when the event
-   * is timed, or not if the event is all day.
-   */
-  if (gcal_event_get_all_day (self->event))
-    {
-      display_text = escaped_text;
-    }
+  /* Set the text to be displayed */
+  if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
+    display_text = get_visible_text (widget, GTK_ORIENTATION_HORIZONTAL);
   else
     {
-      GDateTime *local_start_time;
-      gchar *start_time;
-      local_start_time = g_date_time_to_local (gcal_event_get_date_start (self->event));
-
-      if (self->clock_format_24h)
-        start_time = g_date_time_format (local_start_time, "%R");
+      if (get_vertical_text_height (widget) >= height)
+        display_text = get_visible_text (widget, GTK_ORIENTATION_HORIZONTAL);
       else
-        start_time = g_date_time_format (local_start_time, "%I:%M %P");
-
-      /*
-       * When in horizontal, draw the elements sequentially; otherwise,
-       * draw the time in the first line, and the title in subsequent
-       * lines.
-       */
-      if (self->orientation == GTK_ORIENTATION_HORIZONTAL)
-        {
-          if (is_ltr)
-            display_text = g_strdup_printf ("(%s) %s", start_time, escaped_text);
-          else
-            display_text = g_strdup_printf ("%s (%s)", escaped_text, start_time);
-        }
-      else
-        {
-          display_text = g_strdup_printf ("<b>%s</b>\n%s", start_time, escaped_text);
-        }
-
-      g_clear_pointer (&local_start_time, g_date_time_unref);
-      g_clear_pointer (&escaped_text, g_free);
-      g_clear_pointer (&start_time, g_free);
+        display_text = get_visible_text (widget, GTK_ORIENTATION_VERTICAL);
     }
 
   /* Retrieve the font description */
@@ -660,7 +868,6 @@ gcal_event_widget_button_press_event (GtkWidget      *widget,
 
   self = GCAL_EVENT_WIDGET (widget);
   self->button_pressed = TRUE;
-
   return TRUE;
 }
 
@@ -841,7 +1048,7 @@ gcal_event_widget_set_read_only (GcalEventWidget *event,
                            0,
                            NULL,
                            0,
-                           GDK_ACTION_COPY);
+                           GDK_ACTION_MOVE);
 
       gtk_drag_source_add_text_targets (widget);
     }
