@@ -16,6 +16,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GcalEvent"
+
 #include "gcal-event.h"
 #include "gcal-utils.h"
 
@@ -48,9 +50,16 @@ struct _GcalEvent
 
   ECalComponent      *component;
   ESource            *source;
+
+  gboolean            is_valid : 1;
+  gboolean            is_initialized : 1;
+  GError             *initialization_error;
 };
 
-G_DEFINE_TYPE (GcalEvent, gcal_event, G_TYPE_OBJECT)
+static void          gcal_event_initable_iface_init              (GInitableIface *iface);
+
+G_DEFINE_TYPE_WITH_CODE (GcalEvent, gcal_event, G_TYPE_OBJECT,
+                         G_IMPLEMENT_INTERFACE (G_TYPE_INITABLE, gcal_event_initable_iface_init))
 
 enum {
   PROP_0,
@@ -187,6 +196,7 @@ gcal_event_update_uid_internal (GcalEvent *self)
                                    id->uid);
     }
 
+  e_cal_component_free_id (id);
   g_object_notify (G_OBJECT (self), "uid");
 }
 
@@ -237,6 +247,23 @@ gcal_event_set_component_internal (GcalEvent     *self,
 
       /* Setup start date */
       e_cal_component_get_dtstart (component, &start);
+
+      /*
+       * A NULL start date is invalid. We set something bogus to procceed, and make
+       * it set a GError and return NULL.
+       */
+      if (!start.value)
+        {
+          self->is_valid = FALSE;
+          g_set_error (&self->initialization_error,
+                       GCAL_EVENT_ERROR,
+                       GCAL_EVENT_ERROR_INVALID_START_DATE,
+                       "Event '%s' has an invalid start date", gcal_event_get_uid (self));
+
+          start.value = g_new0 (icaltimetype, 1);
+          *start.value = icaltime_today ();
+        }
+
       date = icaltime_normalize (*start.value);
       zone_start = get_timezone_from_ical (&start);
       date_start = g_date_time_new (zone_start,
@@ -300,6 +327,44 @@ gcal_event_set_component_internal (GcalEvent     *self,
     }
 }
 
+/*
+ * GInitable iface implementation
+ */
+
+G_LOCK_DEFINE_STATIC (init_lock);
+
+static gboolean
+gcal_event_initable_init (GInitable     *initable,
+                          GCancellable  *cancellable,
+                          GError       **error)
+{
+  GcalEvent *self;
+
+  self = GCAL_EVENT (initable);
+
+  G_LOCK (init_lock);
+
+  if (self->is_initialized)
+    goto out;
+
+
+out:
+  self->is_initialized = TRUE;
+
+  if (!self->is_valid)
+    g_propagate_error (error, g_error_copy (self->initialization_error));
+
+  G_UNLOCK (init_lock);
+
+  return self->is_valid;
+}
+
+static void
+gcal_event_initable_iface_init (GInitableIface *iface)
+{
+  iface->init = gcal_event_initable_init;
+}
+
 static void
 gcal_event_finalize (GObject *object)
 {
@@ -309,6 +374,9 @@ gcal_event_finalize (GObject *object)
   g_clear_pointer (&self->dt_end, g_date_time_unref);
   g_clear_pointer (&self->timezone, g_time_zone_unref);
   g_clear_pointer (&self->description, g_free);
+  g_clear_pointer (&self->alarms, g_hash_table_unref);
+  g_clear_pointer (&self->uid, g_free);
+  g_clear_pointer (&self->color, gdk_rgba_free);
   g_clear_object (&self->component);
   g_clear_object (&self->source);
 
@@ -593,27 +661,44 @@ gcal_event_init (GcalEvent *self)
 
   /* Alarms */
   self->alarms = g_hash_table_new_full (g_direct_hash, g_direct_equal, NULL, g_free);
+
+  self->is_valid = TRUE;
+}
+
+/**
+ * gcal_event_error_quark:
+ *
+ * Error quark for event creating.
+ */
+GQuark
+gcal_event_error_quark (void)
+{
+  return g_quark_from_static_string ("Invalid start date");
 }
 
 /**
  * gcal_event_new:
  * @source: (nullable): an #ESource
  * @component: a #ECalComponent
+ * @error: (nullable): return location for a #GError
  *
  * Creates a new event which belongs to @source and
  * is represented by @component. New events will have
  * a %NULL @source.
  *
- * Returns: (transfer full): a #GcalEvent
+ * Returns: (transfer full)(nullable): a #GcalEvent
  */
 GcalEvent*
-gcal_event_new (ESource       *source,
-                ECalComponent *component)
+gcal_event_new (ESource        *source,
+                ECalComponent  *component,
+                GError        **error)
 {
-  return g_object_new (GCAL_TYPE_EVENT,
-                       "source", source,
-                       "component", component,
-                       NULL);
+  return g_initable_new (GCAL_TYPE_EVENT,
+                         NULL,
+                         error,
+                         "source", source,
+                         "component", component,
+                         NULL);
 }
 
 /**
@@ -853,6 +938,7 @@ gcal_event_set_description (GcalEvent   *self,
       ECalComponentText text_component;
       GSList list;
 
+      g_clear_pointer (&self->description, g_free);
       self->description = g_strdup (description);
 
       text_component.value = description;
@@ -1277,34 +1363,39 @@ gcal_event_get_uid (GcalEvent *self)
 gboolean
 gcal_event_is_multiday (GcalEvent *self)
 {
-  GDateTime *end_date;
+  GDateTime *end_date, *real_end_date;
+  gboolean is_multiday;
   gint n_days;
 
   g_return_val_if_fail (GCAL_IS_EVENT (self), FALSE);
 
   end_date = gcal_event_get_date_end (self);
-  n_days = g_date_time_difference (end_date, self->dt_start) / G_TIME_SPAN_DAY;
+  real_end_date = g_date_time_add_seconds (end_date, -1);
+  is_multiday = FALSE;
+  n_days = g_date_time_difference (real_end_date, self->dt_start) / G_TIME_SPAN_DAY;
 
   /*
    * An all-day event with only 1 day of time span is treated as a
    * single-day event.
    */
   if (self->all_day && n_days == 1)
-    return FALSE;
+    is_multiday = FALSE;
 
   /*
    * If any of the following fields are different, we're certain that
    * it's a multiday event. Otherwise, we're certain it's NOT a multiday
    * event.
    */
-  if (g_date_time_get_year (self->dt_start) != g_date_time_get_year (end_date) ||
-      g_date_time_get_month (self->dt_start) != g_date_time_get_month (end_date) ||
-      g_date_time_get_day_of_month (self->dt_start) != g_date_time_get_day_of_month (end_date))
+  if (g_date_time_get_year (self->dt_start) != g_date_time_get_year (real_end_date) ||
+      g_date_time_get_month (self->dt_start) != g_date_time_get_month (real_end_date) ||
+      g_date_time_get_day_of_month (self->dt_start) != g_date_time_get_day_of_month (real_end_date))
     {
-      return TRUE;
+      is_multiday = TRUE;
     }
 
-  return FALSE;
+  g_clear_pointer (&real_end_date, g_date_time_unref);
+
+  return is_multiday;
 }
 
 /**
