@@ -16,12 +16,16 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GcalSourceDialog"
+
+#include "gcal-debug.h"
 #include "gcal-source-dialog.h"
 #include "gcal-utils.h"
 
 #include <glib/gi18n.h>
 #include <goa/goa.h>
 #include <libedataserverui/libedataserverui.h>
+#include <libsoup/soup.h>
 
 struct _GcalSourceDialog
 {
@@ -388,7 +392,7 @@ calendar_listbox_sort_func (GtkListBoxRow *row1,
 
       // If they have the same parent names, compare by the source display names
       if (retval == 0)
-        retval = g_strcmp0 (e_source_get_display_name (source1), e_source_get_display_name (source2));
+        retval = g_ascii_strcasecmp (e_source_get_display_name (source1), e_source_get_display_name (source2));
 
       g_free (parent_name1);
       g_free (parent_name2);
@@ -567,7 +571,7 @@ source_color_changed (GObject    *source,
                       GtkImage *icon)
 {
   ESourceSelectable *extension;
-  GdkPixbuf *pixbuf;
+  cairo_surface_t *surface;
   GdkRGBA out_color;
 
   extension = E_SOURCE_SELECTABLE (source);
@@ -580,9 +584,9 @@ source_color_changed (GObject    *source,
 
   if (gdk_rgba_parse (&out_color, e_source_selectable_get_color (extension)))
     {
-      pixbuf = get_circle_pixbuf_from_color (&out_color, 24);
-      gtk_image_set_from_pixbuf (GTK_IMAGE (icon), pixbuf);
-      g_object_unref (pixbuf);
+      surface = get_circle_surface_from_color (&out_color, 24);
+      gtk_image_set_from_surface (GTK_IMAGE (icon), surface);
+      g_clear_pointer (&surface, cairo_surface_destroy);
     }
 }
 
@@ -608,10 +612,10 @@ static GtkWidget*
 make_row_from_source (GcalSourceDialog *dialog,
                       ESource          *source)
 {
+  cairo_surface_t *surface;
   GtkBuilder *builder;
   GtkWidget *bottom_label;
   GtkWidget *top_label;
-  GdkPixbuf *pixbuf;
   GtkWidget *icon;
   GtkWidget *row;
   GdkRGBA color;
@@ -630,9 +634,9 @@ make_row_from_source (GcalSourceDialog *dialog,
 
   /* source color icon */
   get_color_name_from_source (source, &color);
-  pixbuf = get_circle_pixbuf_from_color (&color, 24);
+  surface = get_circle_surface_from_color (&color, 24);
   icon = GTK_WIDGET (gtk_builder_get_object (builder, "icon"));
-  gtk_image_set_from_pixbuf (GTK_IMAGE (icon), pixbuf);
+  gtk_image_set_from_surface (GTK_IMAGE (icon), surface);
 
   /* source name label */
   top_label = GTK_WIDGET (gtk_builder_get_object (builder, "title"));
@@ -648,8 +652,8 @@ make_row_from_source (GcalSourceDialog *dialog,
   bottom_label = GTK_WIDGET (gtk_builder_get_object (builder, "subtitle"));
   gtk_label_set_label (GTK_LABEL (bottom_label), parent_name);
 
+  g_clear_pointer (&surface, cairo_surface_destroy);
   g_object_unref (builder);
-  g_object_unref (pixbuf);
   g_free (parent_name);
 
   return row;
@@ -939,7 +943,7 @@ stack_visible_child_name_changed (GObject    *object,
 
       // enabled check
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->calendar_visible_check),
-                                    gcal_manager_source_enabled (self->manager, self->source));
+                                    is_source_enabled (self->source));
 
       /* default source check button */
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (self->default_check), (self->source == default_source));
@@ -1238,13 +1242,17 @@ static gboolean
 validate_url_cb (GcalSourceDialog *dialog)
 {
   ESourceAuthentication *auth;
+  ENamedParameters *credentials;
   ESourceExtension *ext;
   ESourceWebdav *webdav;
   ESource *source;
+  SoupURI *soup_uri;
+  const gchar *uri;
   gchar *host, *path;
-  gboolean uri_valid;
+  gboolean uri_valid, is_file;
 
   dialog->validate_url_resource_id = 0;
+  soup_uri = NULL;
   host = path = NULL;
 
   /**
@@ -1265,14 +1273,15 @@ validate_url_cb (GcalSourceDialog *dialog)
   // Clear the entry icon
   gtk_entry_set_icon_from_icon_name (GTK_ENTRY (dialog->calendar_address_entry), GTK_ENTRY_ICON_SECONDARY, NULL);
 
-  // Get the hostname and file path from the server
-  uri_valid = uri_get_fields (gtk_entry_get_text (GTK_ENTRY (dialog->calendar_address_entry)), NULL, &host, &path);
+  /* Get the hostname and file path from the server */
+  uri = gtk_entry_get_text (GTK_ENTRY (dialog->calendar_address_entry));
+  uri_valid = uri_get_fields (uri, NULL, &host, &path, &is_file);
 
-  g_debug ("[source-dialog] host: '%s', path: '%s'", host, path);
-
-  if (host == NULL || !uri_valid)
+  if (!host || !uri_valid)
     goto out;
 
+  g_debug ("Detected host: '%s', path: '%s'", host, path);
+
   /**
    * Create the new source and add the needed
    * extensions.
@@ -1287,82 +1296,91 @@ validate_url_cb (GcalSourceDialog *dialog)
   auth = e_source_get_extension (source, E_SOURCE_EXTENSION_AUTHENTICATION);
   e_source_authentication_set_host (auth, host);
 
-  // Webdav
+  /* Webdav */
+  soup_uri = soup_uri_new (uri);
   webdav = e_source_get_extension (source, E_SOURCE_EXTENSION_WEBDAV_BACKEND);
-  e_source_webdav_set_resource_path (webdav, path);
+  e_source_webdav_set_soup_uri (webdav, soup_uri);
 
   /*
    * If we're dealing with an absolute file path,
    * there is no need to check the server for more
    * sources.
    */
-  if (g_str_has_suffix (path, ".ics"))
+  if (is_file)
     {
       // Set the private source so it saves at closing
       dialog->remote_sources = g_list_append (dialog->remote_sources, source);
 
       // Update buttons
       gtk_widget_set_sensitive (dialog->add_button, source != NULL);
+
+      goto out;
+    }
+
+  /* Pulse the entry while it performs the check */
+  dialog->calendar_address_id = g_timeout_add (ENTRY_PROGRESS_TIMEOUT, (GSourceFunc) pulse_web_entry, dialog);
+
+  /*
+   * Try to retrieve the sources without prompting
+   * username and password. If we get any error,
+   * then it prompts and retry.
+   */
+  credentials = e_named_parameters_new ();
+
+  if (!dialog->prompt_password)
+    {
+      g_debug ("Trying to connect without credentials...");
+
+      /* NULL credentials */
+      e_named_parameters_set (credentials, E_SOURCE_CREDENTIAL_USERNAME, NULL);
+      e_named_parameters_set (credentials, E_SOURCE_CREDENTIAL_PASSWORD, NULL);
+
+      e_webdav_discover_sources (source,
+                                 uri,
+                                 E_WEBDAV_DISCOVER_SUPPORTS_EVENTS,
+                                 credentials,
+                                 NULL,
+                                 discover_sources_cb,
+                                 dialog);
     }
   else
     {
-      ENamedParameters *credentials;
+      gint response;
+      gchar *user, *password;
 
-      // Pulse the entry while it performs the check
-      dialog->calendar_address_id = g_timeout_add (ENTRY_PROGRESS_TIMEOUT, (GSourceFunc) pulse_web_entry, dialog);
+      g_debug ("No credentials failed, retrying with user credentials...");
+
+      user = password = NULL;
+      response = prompt_credentials (dialog, &user, &password);
 
       /*
-       * Try to retrieve the sources without prompting
-       * username and password. If we get any error,
-       * then it prompts and retry.
+       * User entered username and password, let's try
+       * with it.
        */
-      credentials = e_named_parameters_new ();
-
-      if (!dialog->prompt_password)
+      if (response == GTK_RESPONSE_OK)
         {
-          g_debug ("[source-dialog] Trying to connect without credentials...");
+          /* User inputted credentials */
+          e_named_parameters_set (credentials, E_SOURCE_CREDENTIAL_USERNAME, user);
+          e_named_parameters_set (credentials, E_SOURCE_CREDENTIAL_PASSWORD, password);
 
-          // NULL credentials
-          e_named_parameters_set (credentials, E_SOURCE_CREDENTIAL_USERNAME, NULL);
-          e_named_parameters_set (credentials, E_SOURCE_CREDENTIAL_PASSWORD, NULL);
-
-          e_webdav_discover_sources (source, gtk_entry_get_text (GTK_ENTRY (dialog->calendar_address_entry)),
-                                     E_WEBDAV_DISCOVER_SUPPORTS_EVENTS, credentials, NULL, discover_sources_cb,
+          e_webdav_discover_sources (source,
+                                     uri,
+                                     E_WEBDAV_DISCOVER_SUPPORTS_EVENTS,
+                                     credentials,
+                                     NULL,
+                                     discover_sources_cb,
                                      dialog);
         }
-      else
-        {
-          gint response;
-          gchar *user, *password;
 
-          g_debug ("[source-dialog] No credentials failed, retrying with user credentials...");
 
-          user = password = NULL;
-          response = prompt_credentials (dialog, &user, &password);
-
-          /*
-           * User entered username and password, let's try
-           * with it.
-           */
-          if (response == GTK_RESPONSE_OK)
-            {
-              // User inputted credentials
-              e_named_parameters_set (credentials, E_SOURCE_CREDENTIAL_USERNAME, user);
-              e_named_parameters_set (credentials, E_SOURCE_CREDENTIAL_PASSWORD, password);
-
-              e_webdav_discover_sources (source, gtk_entry_get_text (GTK_ENTRY (dialog->calendar_address_entry)),
-                                         E_WEBDAV_DISCOVER_SUPPORTS_EVENTS, credentials, NULL, discover_sources_cb,
-                                         dialog);
-            }
-
-          g_free (user);
-          g_free (password);
-        }
-
-      e_named_parameters_free (credentials);
+      g_free (user);
+      g_free (password);
     }
 
+  e_named_parameters_free (credentials);
+
 out:
+  g_clear_pointer (&soup_uri, soup_uri_free);
   g_free (host);
   g_free (path);
 
@@ -1484,7 +1502,7 @@ check_activated_cb (GtkWidget  *check,
   source = g_object_get_data (G_OBJECT (row), "source");
 
   if (gtk_toggle_button_get_active (GTK_TOGGLE_BUTTON (check)))
-    self->remote_sources = g_list_append (self->remote_sources, source);
+    self->remote_sources = g_list_prepend (self->remote_sources, source);
   else
     self->remote_sources = g_list_remove (self->remote_sources, source);
 
@@ -1497,7 +1515,6 @@ discover_sources_cb (GObject      *source,
                      gpointer      user_data)
 {
   GcalSourceDialog *self;
-  EWebDAVDiscoveredSource *src;
   GSList *discovered_sources, *user_addresses, *aux;
   GError *error;
 
@@ -1525,7 +1542,10 @@ discover_sources_cb (GObject      *source,
        * really want to retry things on unavailable
        * servers.
        */
-      if (!self->prompt_password && error->code == 14)
+      if (!self->prompt_password &&
+          (error->code == 14 ||
+           error->code == 403 ||
+           error->code == 405))
         {
           self->prompt_password = TRUE;
 
@@ -1555,16 +1575,18 @@ discover_sources_cb (GObject      *source,
   /* TODO: show a list of calendars */
   for (aux = discovered_sources; aux != NULL; aux = aux->next)
     {
+      EWebDAVDiscoveredSource *discovered_source;
       gchar *resource_path = NULL;
       gboolean uri_valid;
 
-      src = aux->data;
+      discovered_source = aux->data;
 
-      // Get the new resource path from the uri
-      uri_valid = uri_get_fields (src->href, NULL, NULL, &resource_path);
+      /* Get the new resource path from the uri */
+      uri_valid = uri_get_fields (discovered_source->href, NULL, NULL, &resource_path, NULL);
 
       if (uri_valid)
         {
+          ESourceSelectable *selectable;
           ESourceWebdav *webdav;
           GtkWidget *row, *check;
           ESource *new_source;
@@ -1572,20 +1594,24 @@ discover_sources_cb (GObject      *source,
 
           /* build up the new source */
           new_source = duplicate_source (E_SOURCE (source));
-          e_source_set_display_name (E_SOURCE (new_source), src->display_name);
+          e_source_set_display_name (E_SOURCE (new_source), discovered_source->display_name);
 
           webdav = e_source_get_extension (new_source, E_SOURCE_EXTENSION_WEBDAV_BACKEND);
           e_source_webdav_set_resource_path (webdav, resource_path);
-          e_source_webdav_set_display_name (webdav, src->display_name);
+          e_source_webdav_set_display_name (webdav, discovered_source->display_name);
 
           if (user_addresses)
             e_source_webdav_set_email_address (webdav, user_addresses->data);
 
+          /* Setup the color */
+          selectable = e_source_get_extension (new_source, E_SOURCE_EXTENSION_CALENDAR);
+          e_source_selectable_set_color (selectable, discovered_source->color);
+
           /* create the new row */
           row = gtk_list_box_row_new ();
 
           check = gtk_check_button_new ();
-          gtk_button_set_label (GTK_BUTTON (check), src->display_name);
+          gtk_button_set_label (GTK_BUTTON (check), discovered_source->display_name);
           g_signal_connect (check, "notify::active", G_CALLBACK (check_activated_cb), user_data);
 
           gtk_container_add (GTK_CONTAINER (row), check);
@@ -1593,9 +1619,6 @@ discover_sources_cb (GObject      *source,
 
           g_object_set_data (G_OBJECT (row), "parent-source", source);
           g_object_set_data (G_OBJECT (row), "source", new_source);
-          g_object_set_data_full (G_OBJECT (row), "source-url", g_strdup (src->href), g_free);
-          g_object_set_data_full (G_OBJECT (row), "source-color", g_strdup (src->color), g_free);
-          g_object_set_data_full (G_OBJECT (row), "source-display-name", g_strdup (src->display_name), g_free);
 
           gtk_widget_show_all (row);
         }
@@ -1681,8 +1704,10 @@ notification_child_revealed_changed (GtkWidget  *notification,
         {
           g_warning ("[source-dialog] Error removing source: %s", error->message);
 
-          add_source (self->manager, self->removed_source,
-                      gcal_manager_source_enabled (self->manager, self->removed_source), user_data);
+          add_source (self->manager,
+                      self->removed_source,
+                      is_source_enabled (self->removed_source),
+                      user_data);
 
           gcal_manager_enable_source (self->manager, self->removed_source);
 
@@ -1710,8 +1735,10 @@ undo_remove_action (GtkButton *button,
       // Enable the source before adding it again
       gcal_manager_enable_source (self->manager, self->removed_source);
 
-      add_source (self->manager, self->removed_source,
-                  gcal_manager_source_enabled (self->manager, self->removed_source), user_data);
+      add_source (self->manager,
+                  self->removed_source,
+                  is_source_enabled (self->removed_source),
+                  user_data);
 
       /*
        * Don't clear the pointer, since we don't
@@ -2141,16 +2168,24 @@ goa_account_removed_cb (GoaClient *client,
 }
 
 static void
-goa_client_ready_cb (GcalSourceDialog *dialog,
-                     GoaClient        *client,
-                     gpointer          user_data)
+loading_changed_cb (GcalSourceDialog *dialog)
 {
+  GoaClient *client;
   GList *accounts, *l;
 
+  GCAL_ENTRY;
+
   g_return_if_fail (GCAL_IS_SOURCE_DIALOG (dialog));
-  g_return_if_fail (GOA_IS_CLIENT (client));
+
+  if (gcal_manager_get_loading (dialog->manager))
+    {
+      GCAL_TRACE_MSG ("Not loaded yet");
+      GCAL_EXIT;
+      return;
+    }
 
   /* Add already fetched accounts */
+  client = gcal_manager_get_goa_client (dialog->manager);
   accounts = goa_client_get_accounts (client);
 
   for (l = accounts; l != NULL; l = l->next)
@@ -2160,7 +2195,12 @@ goa_client_ready_cb (GcalSourceDialog *dialog,
   g_signal_connect (client, "account-added", G_CALLBACK (goa_account_added_cb), dialog);
   g_signal_connect (client, "account-removed", G_CALLBACK (goa_account_removed_cb), dialog);
 
+  /* Once we loaded, no need to track it down again */
+  g_signal_handlers_disconnect_by_func (dialog->manager, loading_changed_cb, dialog);
+
   g_list_free (accounts);
+
+  GCAL_EXIT;
 }
 
 /**
@@ -2177,27 +2217,21 @@ gcal_source_dialog_set_manager (GcalSourceDialog *dialog,
 {
   dialog->manager = manager;
 
-  /*
-   * If the GoaClient is already loaded, fetch the online accounts
-   * directly. Otherwise, wait for it to be ready.
-   */
-  if (gcal_manager_is_goa_client_ready (manager))
-    {
-      goa_client_ready_cb (dialog, gcal_manager_get_goa_client (manager), NULL);
-    }
-  else
-    {
-      g_signal_connect_swapped (manager, "goa-client-ready", G_CALLBACK (goa_client_ready_cb), dialog);
-    }
-
-  if (gcal_manager_load_completed (dialog->manager))
+  if (!gcal_manager_get_loading (dialog->manager))
     {
       GList *sources, *l;
 
       sources = gcal_manager_get_sources_connected (dialog->manager);
 
       for (l = sources; l != NULL; l = l->next)
-        add_source (dialog->manager, l->data, gcal_manager_source_enabled (dialog->manager, l->data), dialog);
+        add_source (dialog->manager, l->data, is_source_enabled (l->data), dialog);
+    }
+  else
+    {
+      g_signal_connect_swapped (manager,
+                                "notify::loading",
+                                G_CALLBACK (loading_changed_cb),
+                                dialog);
     }
 
   g_signal_connect (dialog->manager, "source-added", G_CALLBACK (add_source), dialog);
