@@ -73,10 +73,10 @@ gchar*          gcal_get_weekday                                (gint
 
 gchar*          gcal_get_month_name                             (gint                   i);
 
-GdkPixbuf*      gcal_get_pixbuf_from_color                      (GdkRGBA               *color,
+cairo_surface_t*gcal_get_surface_from_color                     (GdkRGBA               *color,
                                                                  gint                   size);
 
-GdkPixbuf*      get_circle_pixbuf_from_color                    (GdkRGBA               *color,
+cairo_surface_t*get_circle_surface_from_color                   (GdkRGBA               *color,
                                                                  gint                   size);
 
 void            get_color_name_from_source                      (ESource               *source,
@@ -123,7 +123,8 @@ void            fix_popover_menu_icons                          (GtkPopover
 gboolean        uri_get_fields                                  (const gchar           *uri,
                                                                  gchar                **schema,
                                                                  gchar                **host,
-                                                                 gchar                **path);
+                                                                 gchar                **path,
+                                                                 gboolean              *is_file);
 
 void            get_source_parent_name_color                    (GcalManager           *manager,
                                                                  ESource               *source,
@@ -138,4 +139,6 @@ gint            get_alarm_trigger_minutes                       (GcalEvent
 gboolean        should_change_date_for_scroll                   (gdouble               *scroll_value,
                                                                  GdkEventScroll        *scroll_event);
 
+gboolean        is_source_enabled                               (ESource               *source);
+
 #endif // __GCAL_UTILS_H__
