@@ -42,9 +42,10 @@ struct _GcalViewInterface
                                                         icaltimetype *start_span,
                                                         icaltimetype *end_span);
 
-  /* Time handling related API */
-  icaltimetype*  (*get_initial_date)                   (GcalView    *view);
-  icaltimetype*  (*get_final_date)                     (GcalView    *view);
+  icaltimetype*  (*get_date)                           (GcalView     *view);
+
+  void           (*set_date)                           (GcalView     *view,
+                                                        icaltimetype *date);
 
   /* Marks related API */
   void           (*clear_marks)                        (GcalView    *view);
