@@ -17,6 +17,8 @@
  * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GcalWeekGrid"
+
 #include "gcal-week-grid.h"
 #include "gcal-week-view.h"
 #include "gcal-utils.h"
@@ -53,7 +55,6 @@ struct _GcalWeekGrid
   gboolean            use_24h_format : 1;
 
   icaltimetype       *active_date;
-  icaltimetype       *current_date;
 
   GcalRangeTree      *events;
 
@@ -73,12 +74,6 @@ struct _GcalWeekGrid
 
 G_DEFINE_TYPE (GcalWeekGrid, gcal_week_grid, GTK_TYPE_CONTAINER);
 
-enum
-{
-  PROP_0,
-  PROP_ACTIVE_DATE,
-  LAST_PROP
-};
 
 enum
 {
@@ -87,7 +82,6 @@ enum
 };
 
 static guint signals[LAST_SIGNAL] = { 0, };
-static GParamSpec* properties[LAST_PROP] = { NULL, };
 
 /* ChildData methods */
 static ChildData*
@@ -255,7 +249,6 @@ gcal_week_grid_finalize (GObject *object)
 
   g_clear_pointer (&self->events, gcal_range_tree_unref);
   g_clear_pointer (&self->active_date, g_free);
-  g_clear_pointer (&self->current_date, g_free);
 
   if (self->redraw_timeout_id > 0)
     {
@@ -311,17 +304,7 @@ gcal_week_grid_get_property (GObject    *object,
                              GValue     *value,
                              GParamSpec *pspec)
 {
-  GcalWeekGrid *self = GCAL_WEEK_GRID (object);
-
-  switch (prop_id)
-    {
-    case PROP_ACTIVE_DATE:
-      g_value_set_boxed (value, self->active_date);
-      return;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
+  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 }
 
 static void
@@ -330,21 +313,7 @@ gcal_week_grid_set_property (GObject      *object,
                              const GValue *value,
                              GParamSpec   *pspec)
 {
-  GcalWeekGrid *self = GCAL_WEEK_GRID (object);
-
-  switch (prop_id)
-    {
-    case PROP_ACTIVE_DATE:
-      g_clear_pointer (&self->active_date, g_free);
-      self->active_date = g_value_dup_boxed (value);
-
-      gtk_widget_queue_resize (GTK_WIDGET (self));
-      gtk_widget_queue_draw (GTK_WIDGET (self));
-      return;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
+  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 }
 
 static void
@@ -720,7 +689,7 @@ gcal_week_grid_size_allocate (GtkWidget     *widget,
                                         &margin);
 
           width = column_width / events_at_range - margin.left - margin.right;
-          height = MAX ((data->end - data->start) * minutes_height - margin.top - margin.bottom, natural_height);
+          height = (data->end - data->start) * minutes_height - margin.top - margin.bottom;
           offset = (width + margin.left + margin.right) * widget_index;
 
           if (ltr)
@@ -829,7 +798,7 @@ gcal_week_grid_button_press (GtkWidget      *widget,
 
   gtk_widget_queue_draw (widget);
 
-  return GDK_EVENT_STOP;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -913,8 +882,15 @@ gcal_week_grid_button_release (GtkWidget      *widget,
     }
   else
     {
-      start = g_date_time_add_minutes (week_start, MAX_MINUTES - column * MINUTES_PER_DAY + minute);
-      end = g_date_time_add_minutes (week_start, MAX_MINUTES - column * MINUTES_PER_DAY + (end_cell + 1) * 30);
+      guint rtl_start_cell, rtl_end_cell, rtl_column;
+
+      /* Fix the minute */
+      rtl_column = 6 - column;
+      rtl_start_cell = start_cell + (rtl_column - column) * 48;
+      rtl_end_cell = (rtl_column * MINUTES_PER_DAY + minute) / 30;
+
+      start = g_date_time_add_minutes (week_start, rtl_start_cell * 30);
+      end = g_date_time_add_minutes (week_start, (rtl_end_cell + 1) * 30);
     }
 
   x = round ((column + 0.5) * (alloc.width / 7.0));
@@ -983,7 +959,7 @@ gcal_week_grid_drag_motion (GtkWidget      *widget,
    * aborts the drag with FALSE.
    */
   gdk_drag_status (context,
-                   self->dnd_cell == -1 ? 0 : GDK_ACTION_COPY,
+                   self->dnd_cell == -1 ? 0 : GDK_ACTION_MOVE,
                    time);
 
   gtk_widget_queue_draw (widget);
@@ -1001,6 +977,7 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
   GcalWeekGrid *self;
   g_autoptr (GDateTime) week_start;
   g_autoptr (GDateTime) dnd_date;
+  g_autoptr (GDateTime) new_end;
   GtkWidget *event_widget;
   GcalEvent *event;
   GTimeSpan timespan = 0;
@@ -1012,7 +989,9 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
   drop_cell = get_dnd_cell (widget, x, y);
   event_widget = gtk_drag_get_source_widget (context);
 
-  week_start = dnd_date = NULL;
+  week_start = NULL;
+  dnd_date = NULL;
+  new_end = NULL;
 
   if (!GCAL_IS_EVENT_WIDGET (event_widget))
     return FALSE;
@@ -1036,8 +1015,7 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
    * Calculate the diff between the dropped cell and the event's start date,
    * so we can update the end date accordingly.
    */
-  if (gcal_event_get_date_end (event))
-    timespan = g_date_time_difference (gcal_event_get_date_end (event), gcal_event_get_date_start (event));
+  timespan = g_date_time_difference (gcal_event_get_date_end (event), gcal_event_get_date_start (event));
 
   /*
    * Set the event's start and end dates. Since the event may have a
@@ -1046,12 +1024,10 @@ gcal_week_grid_drag_drop (GtkWidget      *widget,
   gcal_event_set_all_day (event, FALSE);
   gcal_event_set_date_start (event, dnd_date);
 
-  if (gcal_event_get_date_end (event))
-    {
-      g_autoptr (GDateTime) new_end = g_date_time_add (dnd_date, timespan);
 
-      gcal_event_set_date_end (event, new_end);
-    }
+  /* Setup the new end date */
+  new_end = g_date_time_add (dnd_date, timespan);
+  gcal_event_set_date_end (event, new_end);
 
   /* Commit the changes */
   gcal_manager_update_event (self->manager, event);
@@ -1110,13 +1086,6 @@ gcal_week_grid_class_init (GcalWeekGridClass *klass)
   widget_class->drag_leave = gcal_week_grid_drag_leave;
   widget_class->drag_drop = gcal_week_grid_drag_drop;
 
-  properties[PROP_ACTIVE_DATE] = g_param_spec_boxed ("active-date",
-                                                     "Date",
-                                                     "The active selected date",
-                                                     ICAL_TIME_TYPE,
-                                                     G_PARAM_CONSTRUCT | G_PARAM_READWRITE);
-
-
   signals[EVENT_ACTIVATED] = g_signal_new ("event-activated",
                                            GCAL_TYPE_WEEK_GRID,
                                            G_SIGNAL_RUN_FIRST,
@@ -1125,8 +1094,6 @@ gcal_week_grid_class_init (GcalWeekGridClass *klass)
                                            1,
                                            GCAL_TYPE_EVENT_WIDGET);
 
-  g_object_class_install_properties (object_class, G_N_ELEMENTS (properties), properties);
-
   gtk_widget_class_set_css_name (widget_class, "weekgrid");
 }
 
@@ -1177,18 +1144,6 @@ gcal_week_grid_set_use_24h_format (GcalWeekGrid *self,
   self->use_24h_format = use_24h_format;
 }
 
-void
-gcal_week_grid_set_current_date (GcalWeekGrid *self,
-                                 icaltimetype *current_date)
-{
-  g_return_if_fail (GCAL_IS_WEEK_GRID (self));
-
-  g_clear_pointer (&self->current_date, g_free);
-  self->current_date = gcal_dup_icaltime (current_date);
-
-  gtk_widget_queue_draw (GTK_WIDGET (self));
-}
-
 void
 gcal_week_grid_add_event (GcalWeekGrid *self,
                           GcalEvent    *event)
@@ -1297,3 +1252,14 @@ gcal_week_grid_clear_marks (GcalWeekGrid *self)
 
   gtk_widget_queue_draw (GTK_WIDGET (self));
 }
+
+void
+gcal_week_grid_set_date (GcalWeekGrid *self,
+                         icaltimetype *date)
+{
+  g_clear_pointer (&self->active_date, g_free);
+  self->active_date = gcal_dup_icaltime (date);
+
+  gtk_widget_queue_resize (GTK_WIDGET (self));
+  gtk_widget_queue_draw (GTK_WIDGET (self));
+}
