@@ -19,11 +19,14 @@
 
 #include "config.h"
 
-#include "gcal-week-header.h"
-#include "gcal-week-view.h"
+#define G_LOG_DOMAIN "GcalWeekHeader"
+
+#include "gcal-debug.h"
+#include "gcal-event-widget.h"
 #include "gcal-utils.h"
 #include "gcal-view.h"
-#include "gcal-event-widget.h"
+#include "gcal-week-header.h"
+#include "gcal-week-view.h"
 
 #include <glib/gi18n.h>
 #include <string.h>
@@ -67,7 +70,6 @@ struct _GcalWeekHeader
   gint              redraw_timeout_id;
 
   icaltimetype     *active_date;
-  icaltimetype     *current_date;
 
   gint              selection_start;
   gint              selection_end;
@@ -76,40 +78,12 @@ struct _GcalWeekHeader
   GtkSizeGroup     *sizegroup;
 };
 
-static void           header_collapse                       (GcalWeekHeader *self);
-
-static void           header_expand                         (GcalWeekHeader *self);
-
-static void           on_expand_action_activated            (GcalWeekHeader *self,
-                                                             gpointer        user_data);
-
-static void           gcal_week_header_finalize             (GObject *object);
-
-static void           gcal_week_header_get_property         (GObject    *object,
-                                                             guint       prop_id,
-                                                             GValue     *value,
-                                                             GParamSpec *psec);
-
-static void           gcal_week_header_set_property         (GObject      *object,
-                                                             guint         prop_id,
-                                                             const GValue *value,
-                                                             GParamSpec   *pspec);
-
-static void           gcal_week_header_size_allocate        (GtkWidget     *widget,
-                                                             GtkAllocation *alloc);
-
 typedef enum
 {
   UP,
   DOWN
 } MoveDirection;
 
-enum
-{
-  PROP_0,
-  PROP_ACTIVE_DATE
-};
-
 enum
 {
   EVENT_ACTIVATED,
@@ -166,7 +140,7 @@ on_button_pressed (GcalWeekHeader *self,
 
   gtk_widget_queue_draw (GTK_WIDGET (self));
 
-  return GDK_EVENT_STOP;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -387,9 +361,10 @@ update_overflow (GcalWeekHeader *self)
           gtk_label_set_label (GTK_LABEL (label), text);
           g_free (text);
         }
-      else
+      else if (label)
         {
-          g_clear_pointer (&self->overflow_label[i], gtk_widget_destroy);
+          gtk_widget_destroy (label);
+          self->overflow_label[i] = NULL;
         }
     }
 
@@ -785,22 +760,24 @@ add_event_to_grid (GcalWeekHeader *self,
 
 static void
 update_unchanged_events (GcalWeekHeader *self,
-                         icaltimetype   *old_icaldt,
                          icaltimetype   *new_icaldt)
 {
-  GDateTime *old_week_start, *old_week_end;
-  GDateTime *new_week_start, *new_week_end;
+  g_autoptr (GDateTime) new_week_start, new_week_end;
+  g_autoptr (GDateTime) utc_week_start, utc_week_end;
   GList *events_to_update, *l;
   gint weekday;
 
   events_to_update = NULL;
 
-  old_week_start = get_start_of_week (old_icaldt);
-  old_week_end = get_end_of_week (old_icaldt);
-
   new_week_start = get_start_of_week (new_icaldt);
   new_week_end = get_end_of_week (new_icaldt);
 
+  utc_week_start = g_date_time_new_utc (g_date_time_get_year (new_week_start),
+                                        g_date_time_get_month (new_week_start),
+                                        g_date_time_get_day_of_month (new_week_start),
+                                        0, 0, 0);
+  utc_week_end = g_date_time_add_days (utc_week_start, 7);
+
   for (weekday = 0; weekday < 7; weekday++)
     {
       GList *events;
@@ -810,17 +787,36 @@ update_unchanged_events (GcalWeekHeader *self,
       for (l = events; l != NULL; l = l->next)
         {
           g_autoptr (GDateTime) event_start, event_end;
+          GDateTime *week_start, *week_end;
 
-          event_start = g_date_time_to_local (gcal_event_get_date_start (l->data));
-          event_end = g_date_time_to_local (gcal_event_get_date_end (l->data));
+          /*
+           * When the event is all day, we must be careful to compare its dates
+           * against the UTC variants of the week start and end dates.
+           */
+          if (gcal_event_get_all_day (l->data))
+            {
+              event_start = g_date_time_ref (gcal_event_get_date_start (l->data));
+              event_end = g_date_time_ref (gcal_event_get_date_end (l->data));
+
+              week_start = utc_week_start;
+              week_end = utc_week_end;
+            }
+          else
+            {
+              event_start = g_date_time_to_local (gcal_event_get_date_start (l->data));
+              event_end = g_date_time_to_local (gcal_event_get_date_end (l->data));
+
+              week_start = new_week_start;
+              week_end = new_week_end;
+            }
 
           /*
            * Check if the event must be updated. If we're going to the future, updatable events
            * are events that started somewhere in the past, and are still present. If we're
            * going to the past, updatable events are events that started
            */
-          if (g_date_time_compare (event_start, new_week_end) <= 0 &&
-              g_date_time_compare (event_end, new_week_start) > 0 &&
+          if (g_date_time_compare (event_start, week_end) < 0 &&
+              g_date_time_compare (event_end, week_start) > 0 &&
               !g_list_find (events_to_update, l->data))
             {
               events_to_update = g_list_append (events_to_update, l->data);
@@ -835,10 +831,6 @@ update_unchanged_events (GcalWeekHeader *self,
     }
 
   g_clear_pointer (&events_to_update, g_list_free);
-  g_clear_pointer (&new_week_start, g_date_time_unref);
-  g_clear_pointer (&old_week_start, g_date_time_unref);
-  g_clear_pointer (&new_week_end, g_date_time_unref);
-  g_clear_pointer (&old_week_end, g_date_time_unref);
 }
 
 /* Header */
@@ -1045,7 +1037,6 @@ gcal_week_header_finalize (GObject *object)
   gint i;
 
   g_clear_pointer (&self->active_date, g_free);
-  g_clear_pointer (&self->current_date, g_free);
 
   if (self->redraw_timeout_id > 0)
     {
@@ -1063,17 +1054,7 @@ gcal_week_header_get_property (GObject    *object,
                                GValue     *value,
                                GParamSpec *pspec)
 {
-  GcalWeekHeader *self = GCAL_WEEK_HEADER (object);
-
-  switch (prop_id)
-    {
-    case PROP_ACTIVE_DATE:
-      g_value_set_boxed (value, self->active_date);
-      return;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
+  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 }
 
 static void
@@ -1082,41 +1063,7 @@ gcal_week_header_set_property (GObject      *object,
                                const GValue *value,
                                GParamSpec   *pspec)
 {
-  GcalWeekHeader *self = GCAL_WEEK_HEADER (object);
-  icaltimetype *old_date, *new_date;
-
-  switch (prop_id)
-    {
-    case PROP_ACTIVE_DATE:
-      old_date = self->active_date;
-      new_date = g_value_dup_boxed (value);
-
-      /*
-       * If the active date changed, but we're still in the same week,
-       * there's no need to recalculate visible events.
-       */
-      if (old_date && new_date &&
-          old_date->year == new_date->year &&
-          icaltime_week_number (*old_date) == icaltime_week_number (*new_date))
-        {
-          g_free (new_date);
-          break;
-        }
-
-      self->active_date = new_date;
-
-      update_title (self);
-      gtk_widget_queue_draw (GTK_WIDGET (self));
-
-      if (old_date)
-        update_unchanged_events (self, old_date, self->active_date);
-
-      g_clear_pointer (&old_date, g_free);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
-    }
+  G_OBJECT_WARN_INVALID_PROPERTY_ID (object, prop_id, pspec);
 }
 
 static void
@@ -1387,7 +1334,7 @@ gcal_week_header_drag_motion (GtkWidget      *widget,
    * aborts the drag with FALSE.
    */
   gdk_drag_status (context,
-                   self->dnd_cell == -1 ? 0 : GDK_ACTION_COPY,
+                   self->dnd_cell == -1 ? 0 : GDK_ACTION_MOVE,
                    time);
 
   gtk_widget_queue_draw (widget);
@@ -1405,19 +1352,26 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
   GcalWeekHeader *self;
   g_autoptr (GDateTime) week_start;
   g_autoptr (GDateTime) dnd_date;
+  g_autoptr (GDateTime) new_end;
   GDateTime *start_date;
   GDateTime *end_date;
+  GTimeSpan difference;
   GtkWidget *event_widget;
   GcalEvent *event;
+  gboolean turn_all_day;
   gboolean ltr;
   gint drop_cell;
 
+  GCAL_ENTRY;
+
   self = GCAL_WEEK_HEADER (widget);
   ltr = gtk_widget_get_direction (widget) != GTK_TEXT_DIR_RTL;
   drop_cell = get_dnd_cell (widget, x, y);
   event_widget = gtk_drag_get_source_widget (context);
 
-  week_start = dnd_date = NULL;
+  week_start = NULL;
+  dnd_date = NULL;
+  new_end = NULL;
 
   if (!GCAL_IS_EVENT_WIDGET (event_widget))
     return FALSE;
@@ -1431,7 +1385,9 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
   end_date = gcal_event_get_date_end (event);
   week_start = get_start_of_week (self->active_date);
 
-  if (gcal_event_is_multiday (event) && !gcal_event_get_all_day (event))
+  turn_all_day = !gcal_event_is_multiday (event) || gcal_event_get_all_day (event);
+
+  if (!turn_all_day)
     {
       /*
        * The only case where we don't touch the timezone is for
@@ -1451,22 +1407,13 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
                                       g_date_time_get_month (week_start),
                                       g_date_time_get_day_of_month (week_start) + drop_cell,
                                       0, 0, 0);
-
-      gcal_event_set_all_day (event, TRUE);
     }
 
-  /* Since the event may have a NULL end date, so we have to check it here */
-  if (end_date)
-    {
-      g_autoptr (GDateTime) new_end;
-      GTimeSpan difference;
+  /* End date */
+  difference = turn_all_day ? 24 : g_date_time_difference (end_date, start_date) / G_TIME_SPAN_HOUR;
 
-      difference = g_date_time_difference (end_date, start_date);
-      difference /= G_TIME_SPAN_HOUR;
-
-      new_end = g_date_time_add_hours (dnd_date, difference);
-      gcal_event_set_date_end (event, new_end);
-    }
+  new_end = g_date_time_add_hours (dnd_date, difference);
+  gcal_event_set_date_end (event, new_end);
 
   /*
    * Set the start date ~after~ the end date, so we can compare
@@ -1474,6 +1421,9 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
    */
   gcal_event_set_date_start (event, dnd_date);
 
+  if (turn_all_day)
+    gcal_event_set_all_day (event, TRUE);
+
   /* Commit the changes */
   gcal_manager_update_event (self->manager, event);
 
@@ -1484,7 +1434,7 @@ gcal_week_header_drag_drop (GtkWidget      *widget,
 
   gtk_widget_queue_draw (widget);
 
-  return TRUE;
+  GCAL_RETURN (TRUE);
 }
 
 static void
@@ -1516,14 +1466,6 @@ gcal_week_header_class_init (GcalWeekHeaderClass *kclass)
   widget_class->drag_leave = gcal_week_header_drag_leave;
   widget_class->drag_drop = gcal_week_header_drag_drop;
 
-  g_object_class_install_property (object_class,
-                                   PROP_ACTIVE_DATE,
-                                   g_param_spec_boxed ("active-date",
-                                                       "Date",
-                                                       "The active selected date",
-                                                       ICAL_TIME_TYPE,
-                                                       G_PARAM_CONSTRUCT | G_PARAM_READWRITE));
-
   signals[EVENT_ACTIVATED] = g_signal_new ("event-activated",
                                            GCAL_TYPE_WEEK_HEADER,
                                            G_SIGNAL_RUN_FIRST,
@@ -1664,6 +1606,13 @@ gcal_week_header_add_event (GcalWeekHeader *self,
   else
     end = 6;
 
+  /* Sanity checks */
+  if (start > end || start > 6 || end < 0)
+    {
+      g_warning ("Error adding event '%s' to the week header", gcal_event_get_summary (event));
+      return;
+    }
+
   /* Add the event widget to the grid */
   add_event_to_grid (self, event, start, end);
 
@@ -1717,13 +1666,13 @@ gcal_week_header_remove_event (GcalWeekHeader *self,
       if (event_position == -1)
         continue;
 
-      /* Move remaining events up */
-      move_events_at_column (self, UP, weekday, event_position);
-
       /* Remove from the current weekday */
       l = g_list_remove (l, removed_event);
       self->events[weekday] = l;
 
+      /* Move remaining events up */
+      move_events_at_column (self, UP, weekday, event_position);
+
     }
 
   /* Check if we eventually can merge events */
@@ -1764,20 +1713,6 @@ gcal_week_header_get_children_by_uuid (GcalWeekHeader *self,
   return result;
 }
 
-void
-gcal_week_header_set_current_date (GcalWeekHeader *self,
-                                   icaltimetype   *current_date)
-{
-  g_return_if_fail (GCAL_IS_WEEK_HEADER (self));
-
-  g_clear_pointer (&self->current_date, g_free);
-  self->current_date = gcal_dup_icaltime (current_date);
-
-  update_title (self);
-
-  gtk_widget_queue_draw (GTK_WIDGET (self));
-}
-
 GtkSizeGroup*
 gcal_week_header_get_sidebar_size_group (GcalWeekHeader *self)
 {
@@ -1796,3 +1731,35 @@ gcal_week_header_clear_marks (GcalWeekHeader *self)
 
   gtk_widget_queue_draw (GTK_WIDGET (self));
 }
+
+void
+gcal_week_header_set_date (GcalWeekHeader *self,
+                           icaltimetype   *date)
+{
+  icaltimetype *old_date, *new_date;
+
+  old_date = self->active_date;
+  new_date = gcal_dup_icaltime (date);
+
+  /*
+   * If the active date changed, but we're still in the same week,
+   * there's no need to recalculate visible events.
+   */
+  if (old_date && new_date &&
+      old_date->year == new_date->year &&
+      icaltime_week_number (*old_date) == icaltime_week_number (*new_date))
+    {
+      g_free (new_date);
+      return;
+    }
+
+  self->active_date = new_date;
+
+  update_title (self);
+  gtk_widget_queue_draw (GTK_WIDGET (self));
+
+  if (old_date)
+    update_unchanged_events (self, self->active_date);
+
+  g_clear_pointer (&old_date, g_free);
+}
