@@ -18,12 +18,15 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
-#include "gcal-week-view.h"
+#define G_LOG_DOMAIN "GcalWeekView"
+
+#include "gcal-debug.h"
+#include "gcal-event-widget.h"
 #include "gcal-utils.h"
 #include "gcal-view.h"
-#include "gcal-event-widget.h"
 #include "gcal-week-header.h"
 #include "gcal-week-grid.h"
+#include "gcal-week-view.h"
 
 #include <glib/gi18n.h>
 
@@ -37,12 +40,6 @@ static const double dashed [] =
   6.0
 };
 
-enum
-{
-  PROP_0,
-  PROP_DATE
-};
-
 struct _GcalWeekView
 {
   GtkBox          parent;
@@ -65,7 +62,6 @@ struct _GcalWeekView
 
   /* property */
   icaltimetype   *date;
-  icaltimetype   *current_date;
   GcalManager    *manager; /* weak referenced */
 
   guint           scroll_grid_timeout_id;
@@ -73,51 +69,23 @@ struct _GcalWeekView
   gint            clicked_cell;
 };
 
-static void           gcal_week_view_component_added            (ECalDataModelSubscriber *subscriber,
-                                                                 ECalClient              *client,
-                                                                 ECalComponent           *comp);
-
-static void           gcal_week_view_component_modified         (ECalDataModelSubscriber *subscriber,
-                                                                 ECalClient              *client,
-                                                                 ECalComponent           *comp);
-
-static void           gcal_week_view_component_removed          (ECalDataModelSubscriber *subscriber,
-                                                                 ECalClient              *client,
-                                                                 const gchar             *uid,
-                                                                 const gchar             *rid);
-
-static void           gcal_week_view_freeze                     (ECalDataModelSubscriber *subscriber);
-
-static void           gcal_week_view_thaw                       (ECalDataModelSubscriber *subscriber);
-
-static gboolean       gcal_week_view_draw_hours                 (GcalWeekView *self,
-                                                                 cairo_t      *cr,
-                                                                 GtkWidget    *widget);
-
 static void           gcal_view_interface_init                  (GcalViewInterface *iface);
 
 static void           gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface);
 
-static void           gcal_week_view_finalize                   (GObject        *object);
-
-static void           gcal_week_view_set_property               (GObject        *object,
-                                                                 guint           property_id,
-                                                                 const GValue   *value,
-                                                                 GParamSpec     *pspec);
-
-static void           gcal_week_view_get_property               (GObject        *object,
-                                                                 guint           property_id,
-                                                                 GValue         *value,
-                                                                 GParamSpec     *pspec);
-
-static icaltimetype*  gcal_week_view_get_final_date             (GcalView       *view);
-
 enum
 {
   EVENT_ACTIVATED,
   LAST_SIGNAL
 };
 
+enum
+{
+  PROP_0,
+  PROP_DATE,
+  NUM_PROPS
+};
+
 static guint signals[LAST_SIGNAL] = { 0, };
 
 G_DEFINE_TYPE_WITH_CODE (GcalWeekView, gcal_week_view, GTK_TYPE_BOX,
@@ -143,12 +111,13 @@ update_grid_scroll_position (GcalWeekView *self)
   GtkAdjustment *vadjustment;
   gdouble minutes, real_value;
   gdouble max, page, page_increment, value;
+  gboolean dummy;
 
   /* While the scrolled window is not mapped, we keep waiting */
   if (!gtk_widget_get_realized (self->scrolled_window) ||
       !gtk_widget_get_mapped (self->scrolled_window))
     {
-      return G_SOURCE_CONTINUE;
+      GCAL_RETURN (G_SOURCE_CONTINUE);
     }
 
   now = g_date_time_new_now_local ();
@@ -157,7 +126,7 @@ update_grid_scroll_position (GcalWeekView *self)
 
   /* Don't animate when not today */
   if (datetime_compare_date (now, week_start) < 0 || datetime_compare_date (now, week_end) >= 0)
-    goto out;
+    GCAL_GOTO (out);
 
   vadjustment = gtk_scrolled_window_get_vadjustment (GTK_SCROLLED_WINDOW (self->scrolled_window));
   minutes = g_date_time_get_hour (now) * 60 + g_date_time_get_minute (now);
@@ -173,13 +142,14 @@ update_grid_scroll_position (GcalWeekView *self)
   g_signal_emit_by_name (self->scrolled_window,
                          "scroll-child",
                          GTK_SCROLL_PAGE_FORWARD,
-                         FALSE);
+                         FALSE,
+                         &dummy);
 
   gtk_adjustment_set_page_increment (vadjustment, page_increment);
 
 out:
   self->scroll_grid_timeout_id = 0;
-  return G_SOURCE_REMOVE;
+  GCAL_RETURN (G_SOURCE_REMOVE);
 }
 
 static void
@@ -194,58 +164,33 @@ schedule_position_scroll (GcalWeekView *self)
                                                 self);
 }
 
-/**
- * gcal_week_view_get_initial_date:
- *
- * Since: 0.1
- * Return value: the first day of the week
- * Returns: (transfer full): Release with g_free()
- **/
+/* GcalView implementation */
 static icaltimetype*
-gcal_week_view_get_initial_date (GcalView *view)
+gcal_week_view_get_date (GcalView *view)
 {
-  GcalWeekView *self;
-  icaltimetype *new_date;
+  GcalWeekView *self = GCAL_WEEK_VIEW (view);
 
-  g_return_val_if_fail (GCAL_IS_WEEK_VIEW (view), NULL);
-  self = GCAL_WEEK_VIEW(view);
-  new_date = g_new0 (icaltimetype, 1);
-  *new_date = icaltime_from_day_of_year (icaltime_start_doy_week (*(self->date),
-                                                                  self->first_weekday + 1),
-                                         self->date->year);
-  new_date->is_date = 0;
-  new_date->hour = 0;
-  new_date->minute = 0;
-  new_date->second = 0;
-  *new_date = icaltime_set_timezone (new_date, self->date->zone);
-  return new_date;
+  return self->date;
 }
 
-/**
- * gcal_week_view_get_final_date:
- *
- * Since: 0.1
- * Return value: the last day of the week
- * Returns: (transfer full): Release with g_free()
- **/
-static icaltimetype*
-gcal_week_view_get_final_date (GcalView *view)
+static void
+gcal_week_view_set_date (GcalView     *view,
+                         icaltimetype *date)
 {
-  GcalWeekView *self;
-  icaltimetype *new_date;
+  GcalWeekView *self = GCAL_WEEK_VIEW (view);
 
-  g_return_val_if_fail (GCAL_IS_WEEK_VIEW (view), NULL);
-  self = GCAL_WEEK_VIEW(view);
-  new_date = g_new0 (icaltimetype, 1);
-  *new_date = icaltime_from_day_of_year (icaltime_start_doy_week (*(self->date),
-                                                                  self->first_weekday + 1) + 6,
-                                         self->date->year);
-  new_date->is_date = 0;
-  new_date->hour = 23;
-  new_date->minute = 59;
-  new_date->second = 0;
-  *new_date = icaltime_set_timezone (new_date, self->date->zone);
-  return new_date;
+  GCAL_ENTRY;
+
+  g_clear_pointer (&self->date, g_free);
+  self->date = gcal_dup_icaltime (date);
+
+  /* Propagate the new date */
+  gcal_week_grid_set_date (GCAL_WEEK_GRID (self->week_grid), date);
+  gcal_week_header_set_date (GCAL_WEEK_HEADER (self->header), date);
+
+  schedule_position_scroll (self);
+
+  GCAL_EXIT;
 }
 
 static GList*
@@ -271,6 +216,15 @@ gcal_week_view_clear_marks (GcalView *view)
   gcal_week_grid_clear_marks (GCAL_WEEK_GRID (self->week_grid));
 }
 
+static void
+gcal_view_interface_init (GcalViewInterface *iface)
+{
+  iface->get_date = gcal_week_view_get_date;
+  iface->set_date = gcal_week_view_set_date;
+  iface->get_children_by_uuid = gcal_week_view_get_children_by_uuid;
+  iface->clear_marks = gcal_week_view_clear_marks;
+}
+
 static void
 update_hours_sidebar_size (GcalWeekView *self)
 {
@@ -325,6 +279,8 @@ update_hours_sidebar_size (GcalWeekView *self)
   g_object_unref (layout);
 }
 
+
+/* ECalDataModelSubscriber implementation */
 static void
 gcal_week_view_component_added (ECalDataModelSubscriber *subscriber,
                                 ECalClient              *client,
@@ -334,12 +290,16 @@ gcal_week_view_component_added (ECalDataModelSubscriber *subscriber,
 
   GcalEvent *event;
 
+  GCAL_ENTRY;
+
   event = gcal_event_new (e_client_get_source (E_CLIENT (client)), comp, NULL);
 
   if (gcal_event_is_multiday (event) || gcal_event_get_all_day (event))
     gcal_week_header_add_event (GCAL_WEEK_HEADER (self->header), event);
   else
     gcal_week_grid_add_event (GCAL_WEEK_GRID (self->week_grid), event);
+
+  GCAL_EXIT;
 }
 
 static void
@@ -351,6 +311,8 @@ gcal_week_view_component_modified (ECalDataModelSubscriber *subscriber,
   GcalWeekHeader *header;
   gchar *uuid;
 
+  GCAL_ENTRY;
+
   header = GCAL_WEEK_HEADER (self->header);
 
   uuid = get_uuid_from_component (e_client_get_source (E_CLIENT (client)), comp);
@@ -361,6 +323,8 @@ gcal_week_view_component_modified (ECalDataModelSubscriber *subscriber,
   gcal_week_view_component_added (subscriber, client, comp);
 
   g_free (uuid);
+
+  GCAL_EXIT;
 }
 
 static void
@@ -373,6 +337,8 @@ gcal_week_view_component_removed (ECalDataModelSubscriber *subscriber,
   ESource *source;
   gchar *uuid;
 
+  GCAL_ENTRY;
+
   source = e_client_get_source (E_CLIENT (client));
 
   if (rid != NULL)
@@ -384,6 +350,8 @@ gcal_week_view_component_removed (ECalDataModelSubscriber *subscriber,
   gcal_week_grid_remove_event (GCAL_WEEK_GRID (self->week_grid), uuid);
 
   g_free (uuid);
+
+  GCAL_EXIT;
 }
 
 static void
@@ -503,6 +471,70 @@ gcal_week_view_draw_hours (GcalWeekView *self,
   return FALSE;
 }
 
+static void
+gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface)
+{
+  iface->component_added = gcal_week_view_component_added;
+  iface->component_modified = gcal_week_view_component_modified;
+  iface->component_removed = gcal_week_view_component_removed;
+  iface->freeze = gcal_week_view_freeze;
+  iface->thaw = gcal_week_view_thaw;
+}
+
+static void
+gcal_week_view_finalize (GObject       *object)
+{
+  GcalWeekView *self;
+
+  self = GCAL_WEEK_VIEW (object);
+
+  g_clear_pointer (&self->date, g_free);
+
+  /* Chain up to parent's finalize() method. */
+  G_OBJECT_CLASS (gcal_week_view_parent_class)->finalize (object);
+}
+
+static void
+gcal_week_view_set_property (GObject       *object,
+                             guint          property_id,
+                             const GValue  *value,
+                             GParamSpec    *pspec)
+{
+  switch (property_id)
+    {
+    case PROP_DATE:
+      gcal_view_set_date (GCAL_VIEW (object), g_value_get_boxed (value));
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+    }
+}
+
+static void
+gcal_week_view_get_property (GObject       *object,
+                             guint          property_id,
+                             GValue        *value,
+                             GParamSpec    *pspec)
+{
+  GcalWeekView *self;
+
+  g_return_if_fail (GCAL_IS_WEEK_VIEW (object));
+  self = GCAL_WEEK_VIEW (object);
+
+  switch (property_id)
+    {
+    case PROP_DATE:
+      g_value_set_boxed (value, self->date);
+      break;
+
+    default:
+      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
+      break;
+    }
+}
+
 static void
 gcal_week_view_class_init (GcalWeekViewClass *klass)
 {
@@ -544,87 +576,6 @@ gcal_week_view_init (GcalWeekView *self)
   update_hours_sidebar_size (self);
 }
 
-static void
-gcal_view_interface_init (GcalViewInterface *iface)
-{
-  iface->get_initial_date = gcal_week_view_get_initial_date;
-  iface->get_final_date = gcal_week_view_get_final_date;
-  iface->get_children_by_uuid = gcal_week_view_get_children_by_uuid;
-  iface->clear_marks = gcal_week_view_clear_marks;
-}
-
-static void
-gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface)
-{
-  iface->component_added = gcal_week_view_component_added;
-  iface->component_modified = gcal_week_view_component_modified;
-  iface->component_removed = gcal_week_view_component_removed;
-  iface->freeze = gcal_week_view_freeze;
-  iface->thaw = gcal_week_view_thaw;
-}
-
-static void
-gcal_week_view_finalize (GObject       *object)
-{
-  GcalWeekView *self;
-
-  self = GCAL_WEEK_VIEW (object);
-
-  g_clear_pointer (&self->date, g_free);
-
-  /* Chain up to parent's finalize() method. */
-  G_OBJECT_CLASS (gcal_week_view_parent_class)->finalize (object);
-}
-
-static void
-gcal_week_view_set_property (GObject       *object,
-                             guint          property_id,
-                             const GValue  *value,
-                             GParamSpec    *pspec)
-{
-  GcalWeekView *self;
-
-  self = GCAL_WEEK_VIEW (object);
-
-  switch (property_id)
-    {
-    case PROP_DATE:
-      g_clear_pointer (&self->date, g_free);
-      self->date = g_value_dup_boxed (value);
-
-      schedule_position_scroll (self);
-
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-      break;
-    }
-}
-
-static void
-gcal_week_view_get_property (GObject       *object,
-                             guint          property_id,
-                             GValue        *value,
-                             GParamSpec    *pspec)
-{
-  GcalWeekView *self;
-
-  g_return_if_fail (GCAL_IS_WEEK_VIEW (object));
-  self = GCAL_WEEK_VIEW (object);
-
-  switch (property_id)
-    {
-    case PROP_DATE:
-      g_value_set_boxed (value, self->date);
-      break;
-
-    default:
-      G_OBJECT_WARN_INVALID_PROPERTY_ID (object, property_id, pspec);
-      break;
-    }
-}
-
 /* Public API */
 /**
  * gcal_week_view_new:
@@ -686,16 +637,3 @@ gcal_week_view_set_use_24h_format (GcalWeekView *self,
 
   self->use_24h_format = use_24h;
 }
-
-void
-gcal_week_view_set_current_date (GcalWeekView *self,
-                                 icaltimetype *current_date)
-{
-  g_return_if_fail (GCAL_IS_WEEK_VIEW (self));
-
-  g_clear_pointer (&self->current_date, g_free);
-  self->current_date = current_date;
-
-  gcal_week_header_set_current_date (GCAL_WEEK_HEADER (self->header), current_date);
-  gcal_week_grid_set_current_date (GCAL_WEEK_GRID (self->week_grid), current_date);
-}
