@@ -17,6 +17,9 @@
  * along with this program. If not, see <http://www.gnu.org/licenses/>.
  */
 
+#define G_LOG_DOMAIN "GcalYearView"
+
+#include "gcal-debug.h"
 #include "gcal-year-view.h"
 #include "gcal-view.h"
 #include "gcal-utils.h"
@@ -103,11 +106,15 @@ struct _GcalYearView
   /* text direction factors */
   gint          k;
 
-  /* weak ref to current date */
-  icaltimetype *current_date;
-
   /* date property */
   icaltimetype *date;
+
+  /*
+   * Array with the events at every month. Events
+   * that span multiple months are added multiple
+   * times to the array.
+   */
+  GPtrArray    *events[12];
 };
 
 enum {
@@ -125,7 +132,6 @@ enum
 
 static guint signals[NUM_SIGNALS] = { 0, };
 
-static void   reset_sidebar (GcalYearView *year_view);
 static void   gcal_view_interface_init (GcalViewInterface *iface);
 static void   gcal_data_model_subscriber_interface_init (ECalDataModelSubscriberInterface *iface);
 
@@ -161,26 +167,6 @@ get_last_week_of_year_dmy (gint       first_weekday,
   return g_date_get_iso8601_week_of_year (&day_of_year);
 }
 
-static void
-update_date (GcalYearView *year_view,
-             icaltimetype *new_date)
-{
-  gboolean needs_reset = FALSE;
-  if (year_view->date != NULL && icaltime_compare_date (year_view->date, new_date) && year_view->start_selected_date->day != 0)
-    needs_reset = TRUE;
-
-  g_clear_pointer (&year_view->date, g_free);
-  year_view->date = new_date;
-
-  year_view->first_week_of_year = get_last_week_of_year_dmy (year_view->first_weekday,
-                                                             1, G_DATE_JANUARY,  year_view->date->year);;
-  year_view->last_week_of_year = get_last_week_of_year_dmy (year_view->first_weekday,
-                                                            31, G_DATE_DECEMBER, year_view->date->year);
-
-  if (needs_reset)
-    reset_sidebar (year_view);
-}
-
 static void
 event_activated (GcalEventWidget *widget,
                  gpointer         user_data)
@@ -237,7 +223,17 @@ update_selected_dates_from_button_data (GcalYearView *year_view)
     }
   else
     {
-      *(year_view->start_selected_date) = year_view->date ? *(year_view->date) : *(year_view->current_date);
+      g_autofree icaltimetype *current_date;
+      g_autoptr (GDateTime) now;
+
+      now = g_date_time_new_now_local ();
+      current_date = datetime_to_icaltime (now);
+
+      if (year_view->date)
+        *(year_view->start_selected_date) = *year_view->date;
+      else
+        *(year_view->start_selected_date) = *current_date;
+
       year_view->start_selected_date->hour = 0;
       year_view->start_selected_date->minute = 0;
       year_view->start_selected_date->second = 0;
@@ -264,13 +260,17 @@ update_selected_dates_from_button_data (GcalYearView *year_view)
 static void
 update_no_events_page (GcalYearView *year_view)
 {
-  gchar *title;
+  g_autoptr (GDateTime) start_selected_date, now;
   gboolean has_range;
+  gchar *title;
+
+  now = g_date_time_new_now_local ();
+  start_selected_date = icaltime_to_datetime (year_view->start_selected_date);
 
   has_range = (year_view->start_selected_date->day != year_view->end_selected_date->day ||
                year_view->start_selected_date->month != year_view->end_selected_date->month);
 
-  if (icaltime_compare_date (year_view->current_date, year_view->start_selected_date) == 0)
+  if (datetime_compare_date (now, start_selected_date) == 0)
     {
       title = g_strdup_printf ("%s%s", _("Today"), has_range ? "…" : "");
     }
@@ -484,14 +484,19 @@ update_sidebar_headers (GtkListBoxRow *row,
 
   if (before_shift == -1 || before_shift != row_shift)
     {
+      g_autoptr (GDateTime) now, dt;
       GtkWidget *label;
       gchar *label_str;
 
       row_header = gtk_box_new (GTK_ORIENTATION_VERTICAL, 0);
+      now = g_date_time_new_now_local ();
+
       date = *(year_view->start_selected_date);
       icaltime_adjust (&date, row_shift, 0, 0, 0);
 
-      if (icaltime_compare_date (&date, year_view->current_date) == 0)
+      dt = icaltime_to_datetime (&date);
+
+      if (datetime_compare_date (dt, now) == 0)
         label_str = g_strdup (_("Today"));
       else
         label_str = g_strdup_printf ("%s %d", gcal_get_month_name (date.month  - 1), date.day);
@@ -540,6 +545,26 @@ update_sidebar_headers (GtkListBoxRow *row,
   g_clear_pointer (&row_date, g_date_time_unref);
 }
 
+static void
+update_date (GcalYearView *year_view,
+             icaltimetype *new_date)
+{
+  gboolean needs_reset = FALSE;
+  if (year_view->date != NULL && icaltime_compare_date (year_view->date, new_date) && year_view->start_selected_date->day != 0)
+    needs_reset = TRUE;
+
+  g_clear_pointer (&year_view->date, g_free);
+  year_view->date = gcal_dup_icaltime (new_date);
+
+  year_view->first_week_of_year = get_last_week_of_year_dmy (year_view->first_weekday,
+                                                             1, G_DATE_JANUARY,  year_view->date->year);;
+  year_view->last_week_of_year = get_last_week_of_year_dmy (year_view->first_weekday,
+                                                            31, G_DATE_DECEMBER, year_view->date->year);
+
+  if (needs_reset)
+    reset_sidebar (year_view);
+}
+
 static void
 calculate_coord_for_date (GcalYearView *year_view,
                           gint          day,
@@ -603,6 +628,7 @@ calculate_day_month_for_coord (GcalYearView *year_view,
                                gint         *out_month,
                                gboolean     *is_title)
 {
+  GdkPoint *coordinates;
   gint row, column, i, sw, clicked_cell, day, month, columns_or_rows;
   gint number_of_rows;
   gdouble box_side;
@@ -612,6 +638,7 @@ calculate_day_month_for_coord (GcalYearView *year_view,
   box_side = year_view->navigator_grid->box_side;
   number_of_rows = ceil (12.0 / year_view->number_of_columns);
   sw = 1 - 2 * year_view->k;
+  coordinates = year_view->navigator_grid->coordinates;
 
   *is_title = FALSE;
   columns_or_rows = year_view->number_of_columns > number_of_rows ? year_view->number_of_columns : number_of_rows;
@@ -619,16 +646,16 @@ calculate_day_month_for_coord (GcalYearView *year_view,
   for (i = 0; i < columns_or_rows && ((row == -1) || (column == -1)); i++)
     {
       if (row == -1 &&
-          y > year_view->navigator_grid->coordinates[i * year_view->number_of_columns].y &&
-          y < year_view->navigator_grid->coordinates[i * year_view->number_of_columns].y + year_view->row_height)
+          y > coordinates[i * year_view->number_of_columns].y &&
+          y < coordinates[i * year_view->number_of_columns].y + year_view->row_height)
         {
-          if (y < year_view->navigator_grid->coordinates[i * year_view->number_of_columns].y + box_side)
+          if (y < coordinates[i * year_view->number_of_columns].y + box_side)
             *is_title = TRUE;
           row = i;
         }
       if (column == -1 &&
-          x > year_view->navigator_grid->coordinates[i].x + box_side * year_view->show_week_numbers * (1 - year_view->k) &&
-          x < year_view->navigator_grid->coordinates[i].x + year_view->column_width - box_side * year_view->show_week_numbers * year_view->k)
+          x > coordinates[i].x + box_side * year_view->show_week_numbers * (1 - year_view->k) &&
+          x < coordinates[i].x + year_view->column_width - box_side * year_view->show_week_numbers * year_view->k)
         {
           column = i;
         }
@@ -659,6 +686,45 @@ calculate_day_month_for_coord (GcalYearView *year_view,
   *out_day = day;
   return TRUE;
 }
+static guint
+count_events_at_day (GcalYearView *self,
+                     icaltimetype *today)
+{
+  g_autoptr (GDateTime) today_start, today_end;
+  GPtrArray *events;
+  guint i, n_events;
+
+  events = self->events[today->month - 1];
+  n_events = 0;
+  today_start = icaltime_to_datetime (today);
+  today_end = g_date_time_add_days (today_start, 1);
+
+  for (i = 0; i < events->len; i++)
+    {
+      g_autoptr (GDateTime) event_start, event_end;
+      GcalEvent *event;
+
+      event = g_ptr_array_index (events, i);
+
+      event_start = g_date_time_ref (gcal_event_get_date_start (event));
+
+      if (gcal_event_get_all_day (event))
+          event_end = g_date_time_add_days (gcal_event_get_date_end (event), -1);
+      else
+          event_end = g_date_time_ref (gcal_event_get_date_end (event));
+
+
+      if (datetime_compare_date (event_start, today_end) >= 0 ||
+          datetime_compare_date (event_end, today_start) < 0)
+        {
+          continue;
+        }
+
+      n_events++;
+    }
+
+  return n_events;
+}
 
 static void
 draw_month_grid (GcalYearView *year_view,
@@ -673,6 +739,8 @@ draw_month_grid (GcalYearView *year_view,
   PangoLayout *layout, *slayout;
   PangoFontDescription *font_desc, *sfont_desc;
 
+  g_autoptr (GDateTime) now;
+
   GdkRGBA color;
   gint layout_width, layout_height, i, j, sw;
   gint column, row;
@@ -680,8 +748,9 @@ draw_month_grid (GcalYearView *year_view,
   gint days_delay, days, shown_rows, sunday_idx;
   gchar *str, *nr_day, *nr_week;
   gboolean selected_day;
-  GList *events;
-  icaltimetype start_date, end_date;
+  icaltimetype today;
+
+  now = g_date_time_new_now_local ();
 
   cairo_save (cr);
   context = gtk_widget_get_style_context (widget);
@@ -747,18 +816,14 @@ draw_month_grid (GcalYearView *year_view,
   shown_rows = ceil (days / 7.0);
   sunday_idx = year_view->k * 7 + sw * ((7 - year_view->first_weekday) % 7) + year_view->show_week_numbers * (1 - year_view->k);
 
-  start_date.day    = 1;
-  start_date.month  = month_nr + 1;
-  start_date.year   = year_view->date->year;
-  start_date.hour   = 0;
-  start_date.minute = 0;
-  start_date.second = 0;
-  end_date.day    = 1;
-  end_date.month  = month_nr + 1;
-  end_date.year   = year_view->date->year;
-  end_date.hour   = 23;
-  end_date.minute = 59;
-  end_date.second = 59;
+  today = icaltime_today ();
+  today.year = year_view->date->year;
+  today.month = month_nr + 1;
+  today.day = 1;
+  today.hour = 0;
+  today.minute = 0;
+  today.second = 0;
+  today.zone = gcal_manager_get_system_timezone (year_view->manager);
 
   for (i = 0; i < 7 * shown_rows; i++)
     {
@@ -844,8 +909,9 @@ draw_month_grid (GcalYearView *year_view,
             }
         }
 
-      if (year_view->date->year == year_view->current_date->year && month_nr + 1 == year_view->current_date->month &&
-          j == year_view->current_date->day)
+      if (year_view->date->year == g_date_time_get_year (now) &&
+          month_nr + 1 == g_date_time_get_month (now) &&
+          j == g_date_time_get_day_of_month (now))
         {
           PangoLayout *clayout;
           PangoFontDescription *cfont_desc;
@@ -908,23 +974,25 @@ draw_month_grid (GcalYearView *year_view,
                              layout);
         }
 
-      start_date.day = end_date.day = j;
-      events = gcal_manager_get_events (year_view->manager, &start_date, &end_date);
-      if (events != NULL)
+      /* Update the current day, so we can count the events at this day */
+      today.day = j;
+
+      if (count_events_at_day (year_view, &today) > 0)
         {
           gtk_style_context_save (context);
           gtk_style_context_add_class (context, "with-events");
+
           if (selected_day)
             gtk_style_context_add_class (context, "with-events-selected");
           else if (column == sunday_idx)
             gtk_style_context_add_class (context, "with-events-sunday");
-          box_padding_start = (box_side - VISUAL_CLUES_SIDE) / 2 > 0 ? (box_side - VISUAL_CLUES_SIDE) / 2 : 0;
+
+          box_padding_start = MAX (0, (box_side - VISUAL_CLUES_SIDE) / 2);
           gtk_render_background (context, cr,
                                  box_side * column + x + sw * box_padding_start - year_view->k * VISUAL_CLUES_SIDE,
                                  box_side * (row + 1) + y + box_padding_top + layout_height + 2.0,
                                  VISUAL_CLUES_SIDE, VISUAL_CLUES_SIDE);
           gtk_style_context_restore (context);
-          g_list_free_full (events, g_object_unref);
         }
 
       g_free (nr_day);
@@ -1067,7 +1135,7 @@ navigator_button_press_cb (GcalYearView   *year_view,
   gboolean is_title = FALSE;
 
   if (!calculate_day_month_for_coord (year_view, event->x, event->y, &day, &month, &is_title))
-    return FALSE;
+    return GDK_EVENT_PROPAGATE;
 
   if (is_title)
     day = 1;
@@ -1076,7 +1144,7 @@ navigator_button_press_cb (GcalYearView   *year_view,
   year_view->selected_data->start_day = day;
   year_view->selected_data->start_month = month;
 
-  return TRUE;
+  return GDK_EVENT_PROPAGATE;
 }
 
 static gboolean
@@ -1138,7 +1206,7 @@ navigator_motion_notify_cb (GcalYearView   *year_view,
                             GdkEventMotion *event,
                             GtkWidget      *widget)
 {
-  gint day, month;
+  gint day = 0, month = 0;
   gboolean is_title = FALSE;
 
   /* Cancel the hover when selecting a date range */
@@ -1236,10 +1304,7 @@ add_event_clicked_cb (GcalYearView *year_view,
 
   if (year_view->start_selected_date->day == 0)
     {
-      start_date = g_date_time_new_local (year_view->current_date->year,
-                                          year_view->current_date->month,
-                                          year_view->current_date->day,
-                                          0, 0, 0);
+      start_date = g_date_time_new_now_local ();
     }
   else
     {
@@ -1379,7 +1444,7 @@ navigator_drag_motion_cb (GcalYearView   *self,
           self->selected_data->dnd_day = day;
           self->selected_data->dnd_month = month;
 
-          gdk_drag_status (context, GDK_ACTION_COPY, time);
+          gdk_drag_status (context, GDK_ACTION_MOVE, time);
 
           retval = TRUE;
         }
@@ -1480,10 +1545,67 @@ navigator_drag_leave_cb (GcalYearView   *self,
   gtk_widget_queue_draw (navigator);
 }
 
+/* GcalView implementation */
+static icaltimetype*
+gcal_year_view_get_date (GcalView *view)
+{
+  GcalYearView *self = GCAL_YEAR_VIEW (view);
+
+  return self->date;
+}
+
+static void
+gcal_year_view_set_date (GcalView     *view,
+                         icaltimetype *date)
+{
+  GCAL_ENTRY;
+
+  update_date (GCAL_YEAR_VIEW (view), date);
+
+  GCAL_EXIT;
+}
+
+static GList*
+gcal_year_view_get_children_by_uuid (GcalView    *view,
+                                     const gchar *uuid)
+{
+  GcalYearView *year_view = GCAL_YEAR_VIEW (view);
+  GList *children, *l, *result = NULL;
+
+  children = gtk_container_get_children (GTK_CONTAINER (year_view->events_sidebar));
+
+  for (l = children; l != NULL; l = g_list_next (l))
+    {
+      GcalEventWidget *child_widget;
+      GcalEvent *event;
+
+      child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
+      event = gcal_event_widget_get_event (child_widget);
+
+
+      if (child_widget != NULL && g_strcmp0 (uuid, gcal_event_get_uid (event)) == 0)
+        result = g_list_append (result, child_widget);
+    }
+
+  g_list_free (children);
+
+  return result;
+}
+
+static void
+gcal_view_interface_init (GcalViewInterface *iface)
+{
+  /* FIXME: implement what's needed */
+  iface->get_date = gcal_year_view_get_date;
+  iface->set_date = gcal_year_view_set_date;
+  iface->get_children_by_uuid = gcal_year_view_get_children_by_uuid;
+}
+
 static void
 gcal_year_view_finalize (GObject *object)
 {
   GcalYearView *year_view = GCAL_YEAR_VIEW (object);
+  guint i;
 
   g_free (year_view->navigator_grid);
   g_free (year_view->selected_data);
@@ -1495,6 +1617,9 @@ gcal_year_view_finalize (GObject *object)
 
   g_clear_object (&year_view->calendar_settings);
 
+  for (i = 0; i < 12; i++)
+    g_clear_pointer (&year_view->events[i], g_ptr_array_unref);
+
   G_OBJECT_CLASS (gcal_year_view_parent_class)->finalize (object);
 }
 
@@ -1532,7 +1657,7 @@ gcal_year_view_set_property (GObject      *object,
   switch (prop_id)
     {
     case PROP_DATE:
-      update_date (self, g_value_dup_boxed (value));
+      gcal_view_set_date (GCAL_VIEW (self), g_value_get_boxed (value));
       break;
 
     case PROP_SHOW_WEEK_NUMBERS:
@@ -1651,47 +1776,17 @@ gcal_year_view_direction_changed (GtkWidget        *widget,
     year_view->k = 1;
 }
 
-static GList*
-gcal_year_view_get_children_by_uuid (GcalView    *view,
-                                     const gchar *uuid)
-{
-  GcalYearView *year_view = GCAL_YEAR_VIEW (view);
-  GList *children, *l, *result = NULL;
-
-  children = gtk_container_get_children (GTK_CONTAINER (year_view->events_sidebar));
-  for (l = children; l != NULL; l = g_list_next (l))
-    {
-      GcalEventWidget *child_widget;
-      GcalEvent *event;
-
-      child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
-      event = gcal_event_widget_get_event (child_widget);
-
-
-      if (child_widget != NULL && g_strcmp0 (uuid, gcal_event_get_uid (event)) == 0)
-        result = g_list_append (result, child_widget);
-    }
-  g_list_free (children);
-
-  return result;
-}
-
 static void
 gcal_year_view_component_added (ECalDataModelSubscriber *subscriber,
                                 ECalClient              *client,
                                 ECalComponent           *comp)
 {
-  GcalYearView *year_view = GCAL_YEAR_VIEW (subscriber);
-
+  GcalYearView *self = GCAL_YEAR_VIEW (subscriber);
+  GDateTime *event_start, *event_end;
   GcalEvent *event;
   GError *error;
-  GList **days_widgets_array;
   GList *l;
-  gint i, days_span;
-
-  ECalComponentDateTime date;
-  time_t event_start, event_end, range_start, range_end;
-  icaltimezone *zone;
+  guint i, start_month, end_month;
 
   error = NULL;
   event = gcal_event_new (e_client_get_source (E_CLIENT (client)), comp, &error);
@@ -1703,85 +1798,26 @@ gcal_year_view_component_added (ECalDataModelSubscriber *subscriber,
       return;
     }
 
-  update_selected_dates_from_button_data (year_view);
-  days_span = icaltime_day_of_year(*(year_view->end_selected_date)) - icaltime_day_of_year(*(year_view->start_selected_date)) + 1;
-  days_widgets_array = g_new0 (GList*, days_span);
+  event_start = gcal_event_get_date_start (event);
+  event_end = gcal_event_get_date_end (event);
 
-  /* check if event belongs to range */
-  zone = gcal_manager_get_system_timezone (year_view->manager);
-  range_start = icaltime_as_timet_with_zone (*(year_view->start_selected_date), zone);
-  range_end = icaltime_as_timet_with_zone (*(year_view->end_selected_date), zone);
+  /* Calculate the start & end months */
+  start_month = g_date_time_get_month (event_start) - 1;
+  end_month = g_date_time_get_month (event_end) - 1;
 
-  e_cal_component_get_dtstart (comp, &date);
-  event_start = icaltime_as_timet_with_zone (*(date.value), date.value->zone != NULL ? date.value->zone : zone);
-  e_cal_component_free_datetime (&date);
+  if (g_date_time_get_year (event_start) < self->date->year)
+    start_month = 0;
 
-  e_cal_component_get_dtend (comp, &date);
-  if (date.value != NULL)
-    event_end = icaltime_as_timet_with_zone (*(date.value), date.value->zone != NULL ? date.value->zone : zone);
-  else
-    event_end = event_start;
-  e_cal_component_free_datetime (&date);
+  if (g_date_time_get_year (event_end) > self->date->year)
+    end_month = 11;
 
-  if (!((event_start <= range_start && event_end >= range_end) ||
-        (event_start >= range_start && event_end <= range_end) ||
-        (event_start >= range_start && event_start <= range_end) ||
-        (event_end >= range_start && event_end <= range_end)))
-    {
-      goto out;
-    }
+  /* Add the event to the cache */
+  for (i = start_month; i <= end_month; i++)
+    g_ptr_array_add (self->events[i], event);
 
-  add_event_to_day_array (year_view, event, days_widgets_array, days_span);
-  gtk_stack_set_visible_child_name (GTK_STACK (year_view->navigator_stack), "events-list");
+  update_sidebar (self);
 
-  for (i = 0; i < days_span; i++)
-    {
-      GList *current_day = days_widgets_array[i];
-      for (l = current_day; l != NULL; l = g_list_next (l))
-        {
-          GtkWidget *child_widget = l->data;
-          gtk_widget_show (child_widget);
-          g_signal_connect (child_widget, "activate", G_CALLBACK (event_activated), year_view);
-          g_object_set_data (G_OBJECT (child_widget), "shift", GINT_TO_POINTER (i));
-          gtk_container_add (GTK_CONTAINER (year_view->events_sidebar), child_widget);
-        }
-
-      g_list_free (current_day);
-    }
-
-  gtk_widget_queue_draw (year_view->navigator);
-
-out:
-  g_clear_object (&event);
-  g_free (days_widgets_array);
-}
-
-static void
-gcal_year_view_component_changed (ECalDataModelSubscriber *subscriber,
-                                  ECalClient              *client,
-                                  ECalComponent           *comp)
-{
-  GcalYearView *year_view = GCAL_YEAR_VIEW (subscriber);
-  GList *children, *l;
-  gchar *uuid;
-
-  uuid = get_uuid_from_component (e_client_get_source (E_CLIENT (client)), comp);
-  children = gtk_container_get_children (GTK_CONTAINER (year_view->events_sidebar));
-  for (l = children; l != NULL; l = g_list_next (l))
-    {
-      GcalEventWidget *child_widget;
-      GcalEvent *event;
-
-      child_widget = GCAL_EVENT_WIDGET (gtk_bin_get_child (GTK_BIN (l->data)));
-      event = gcal_event_widget_get_event (child_widget);
-
-      if (child_widget != NULL && g_strcmp0 (uuid, gcal_event_get_uid (event)) == 0)
-        gtk_widget_destroy (GTK_WIDGET (l->data));
-    }
-  g_list_free (children);
-  g_free (uuid);
-
-  gcal_year_view_component_added (subscriber, client, comp);
+  gtk_widget_queue_draw (GTK_WIDGET (self->navigator));
 }
 
 static void
@@ -1794,6 +1830,7 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
   GList *children, *l;
   ESource *source;
   gchar *uuid;
+  guint i;
   gint number_of_children;
 
   source = e_client_get_source (E_CLIENT (client));
@@ -1830,10 +1867,48 @@ gcal_year_view_component_removed (ECalDataModelSubscriber *subscriber,
       gtk_stack_set_visible_child_name (GTK_STACK (year_view->navigator_stack), "no-events");
     }
 
+  /* Also remove from the cached list of events */
+  for (i = 0; i < 12; i++)
+    {
+      GPtrArray *events;
+      guint j;
+
+      events = year_view->events[i];
+
+      for (j = 0; j < events->len; j++)
+        {
+          GcalEvent *event;
+
+          event = g_ptr_array_index (events, j);
+
+          if (g_strcmp0 (gcal_event_get_uid (event), uuid) != 0)
+            continue;
+
+          g_ptr_array_remove (events, event);
+        }
+    }
+
+  gtk_widget_queue_draw (GTK_WIDGET (year_view->navigator));
+
   g_list_free (children);
   g_free (uuid);
 }
 
+static void
+gcal_year_view_component_changed (ECalDataModelSubscriber *subscriber,
+                                  ECalClient              *client,
+                                  ECalComponent           *comp)
+{
+  ECalComponentId *id;
+
+  id = e_cal_component_get_id (comp);
+
+  gcal_year_view_component_removed (subscriber, client, id->uid, id->rid);
+  gcal_year_view_component_added (subscriber, client, comp);
+
+  g_clear_pointer (&id, e_cal_component_free_id);
+}
+
 static void
 gcal_year_view_freeze (ECalDataModelSubscriber *subscriber)
 {
@@ -1904,6 +1979,11 @@ gcal_year_view_class_init (GcalYearViewClass *klass)
 static void
 gcal_year_view_init (GcalYearView *self)
 {
+  guint i;
+
+  for (i = 0; i < 12; i++)
+    self->events[i] = g_ptr_array_new ();
+
   gtk_widget_init_template (GTK_WIDGET (self));
 
   if (gtk_widget_get_direction (GTK_WIDGET (self)) == GTK_TEXT_DIR_LTR)
@@ -1936,14 +2016,7 @@ gcal_year_view_init (GcalYearView *self)
                      0,
                      NULL,
                      0,
-                     GDK_ACTION_COPY);
-}
-
-static void
-gcal_view_interface_init (GcalViewInterface *iface)
-{
-  /* FIXME: implement what's needed */
-  iface->get_children_by_uuid = gcal_year_view_get_children_by_uuid;
+                     GDK_ACTION_MOVE);
 }
 
 static void
@@ -1977,14 +2050,3 @@ gcal_year_view_set_use_24h_format (GcalYearView *year_view,
 {
   year_view->use_24h_format = use_24h_format;
 }
-
-void
-gcal_year_view_set_current_date (GcalYearView *year_view,
-                                 icaltimetype *current_date)
-{
-  year_view->current_date = current_date;
-
-  /* Launches update */
-  gtk_widget_queue_draw (GTK_WIDGET (year_view));
-  update_sidebar (year_view);
-}
